/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

#include "utils.h"
#include "LanguageSet.h"

const gchar* SupportedLanguageStr[]={
    "unlabeled",
    "number",
    "symbol",
    "en",
    "jp",
    "kr",
    ZH_SIMPLIFIED_STR,
    ZH_TRADITIONAL_STR,
    NULL
};

LanguageSet* languageSet_new(){
    return g_array_new(FALSE,FALSE,sizeof(SupportedLanguage));
}

gboolean languageSet_has(LanguageSet *self,SupportedLanguage lang){
    int i;
    int len=self->len;
    SupportedLanguage lan;
    for(i=0;i<len;i++){
        lan=g_array_index(self,SupportedLanguage,i);
        if (lan==lang)
            return TRUE;
    }
    return FALSE;
}

guint languageSet_size(LanguageSet *self){
    return self->len;
}

gboolean languageSet_add(LanguageSet *self,SupportedLanguage lang){
    if (languageSet_has(self,lang)){
        return FALSE;
    }
    g_array_append_val(self,lang);
    return TRUE;
}

gboolean languageSet_add_langString(LanguageSet *self,const gchar* langStr){
    SupportedLanguage lang=lang_parse_string(langStr);
    if (lang==LANG_UNLABELED){
        return FALSE;
    }
    return languageSet_add(self,lang);
}

LanguageSet* languageSet_clone(LanguageSet *self){
    LanguageSet *ret=languageSet_new();
    languageSet_copy(self,ret);
    return ret;
}

void languageSet_copy(LanguageSet *self,LanguageSet *src){
    languageSet_reset(self);
    G_ARRAY_COPY(self,src);
}


void languageSet_reset(LanguageSet *self){
    if (self->len>0)
        G_ARRAY_REMOVE_ALL(self);
}


void languageSet_free(LanguageSet *self){
    g_array_free(self,TRUE);
}

gchar* languageSet_to_string(LanguageSet *self){
    GString *strBuf=g_string_new("");
    int i,len=languageSet_size(self);
    SupportedLanguage lang;

    for(i=0;i<len;i++){
	lang=g_array_index(self,SupportedLanguage,i);
	if (i>0){
	    g_string_append(strBuf,", ");
	}
	g_string_append(strBuf,lang_to_string(lang));
    }
    g_string_append(strBuf,"");
    return g_string_free(strBuf,FALSE);
}


/*===================================================================
 * Language supporting functions
 */

SupportedLanguage lang_parse_string(const gchar* str){
     if (!str)
         return LANG_UNLABELED;
     int i;
     for(i=0;SupportedLanguageStr[i]!=NULL;i++){
         if (g_ascii_strcasecmp(SupportedLanguageStr[i],str)==0){
             return i;
         }
     }
     if (g_ascii_strcasecmp(str,"zh_CN")==0){
         return LANG_ZH_SIMPLIFIED;
     }
     if (g_ascii_strcasecmp(str,"zh_TW")==0){
         return LANG_ZH_TRADITIONAL;
     }
     if (g_ascii_strcasecmp(str,"zh_HK")==0){
         return LANG_ZH_TRADITIONAL;
     }
     return LANG_UNLABELED;
}

const gchar* lang_to_string(SupportedLanguage lang){
    return SupportedLanguageStr[lang];
}




