/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

//#include "WritRecogn.h"
#include "RadicalArray.h"
#include "utils.h"

RadicalArray* radicalArray_new(){
    RadicalArray* obj=NEW_INSTANCE(RadicalArray);
    obj->type=RADICAL_TYPE_RADICAL;
    obj->radArray=g_ptr_array_new();
    return obj;
}

RadicalArray* radicalArray_sized_new(guint reserved_size){
    RadicalArray* obj=NEW_INSTANCE(RadicalArray);
    obj->type=RADICAL_TYPE_RADICAL;
    obj->radArray=g_ptr_array_sized_new(reserved_size);
    return obj;
}

RadicalType radicalArray_get_radicalType(RadicalArray *self){
    return self->type;
}

void radicalArray_set_radicalType(RadicalArray *self,RadicalType type){
    self->type=type;
}

gpointer radicalArray_free(RadicalArray *self,gboolean free_segment){
    if (!self)
	return NULL;
    if (free_segment){
        radicalArray_reset(self);
    }
    return g_ptr_array_free(self->radArray,free_segment);
}

void radicalArray_clone(RadicalArray *self){
    RadicalArray *obj=radicalArray_new(self->type);
    radicalArray_copy(obj,self);
}

void radicalArray_copy(RadicalArray *self,RadicalArray *src){
//    g_debug("radicalArray_copy self.len=%d src.len=%d \n",radicalArray_size(self),radicalArray_size(src));
    radicalArray_reset(self);
    self->type=src->type;
    int i,len=radicalArray_size(src);
//    g_debug("radicalArray_copy len=%d\n",len);
    Radical *rad,*new_rad;
    gpointer ret;
    for(i=0;i<len;i++){
//        g_debug("radicalArray_copy i=%d\n",i);
	ret=radicalArray_index(src,i);
	rad=RADICAL(ret);
	new_rad=writrecogn_radical_clone(rad);
	switch(self->type){
	    case RADICAL_TYPE_RADICAL:
		radicalArray_add(self,new_rad);
		break;
	    case RADICAL_TYPE_RAWSTROKE:
		radicalArray_add(self,RAWSTROKE(new_rad));
		break;
	    case RADICAL_TYPE_ABSCHARACTER:
		radicalArray_add(self,ABSCHARACTER(new_rad));
		break;
	    case RADICAL_TYPE_FULLCHARACTER:
		radicalArray_add(self,FULLCHARACTER(new_rad));
		break;
	}
    }
}

void radicalArray_reset(RadicalArray *self){
    int i,len=radicalArray_size(self);
    gpointer ret;
    for(i=0;i<len;i++){
	ret=g_ptr_array_index(self->radArray,i);
        g_object_unref(ret);
    }
    G_PTR_ARRAY_REMOVE_ALL(self->radArray);
}

guint radicalArray_size(RadicalArray *self){
    return self->radArray->len;
}

void radicalArray_add(RadicalArray *self,gpointer obj){
    g_ptr_array_add(self->radArray,obj);
}

gboolean radicalArray_has_radicalCode(RadicalArray *self,RadicalCode code){
    if (radicalArray_find_by_code(self,code))
	return TRUE;
    return FALSE;
}

gpointer radicalArray_index(RadicalArray *self,gint index){
    return g_ptr_array_index(self->radArray,index);
}

gpointer radicalArray_find_by_code(RadicalArray *self,RadicalCode code){
    gpointer *ret=NULL;
    gint i,len=self->radArray->len;
    RadicalCode currCode;
    for(i=0;i<len;i++){
        ret=radicalArray_index(self,i);
	currCode=WRITRECOGN_RADICAL(ret)->radicalCode;
        if (currCode==code){
            return ret;
        }
    }
    return NULL;
}

gboolean radicalArray_is_empty(RadicalArray *self){
    if (!self)
	return TRUE;
    if (radicalArray_size(self)==0){
	return TRUE;
    }
    return FALSE;
}


