/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

/**
 * SECTION: RadicalArray
 * @short_description: RadicalArray stores radicals to GArray
 * @stability: Stable
 * @see_also: #writRecognRadical
 * @include: writrecogn-radical_list.h
 *
 * Unlike RadicalList which is database ended. RadicalArray only deals with GArray.
 *
 * It can be used to pass the universal set in database, 
 * or store the returned results.
 */
#ifndef RADICAL_ARRAY_H_
#define RADICAL_ARRAY_H_
#include <glib.h>
#include "WritRecogn.h"
struct _RadicalArray{
    RadicalType type;
    GPtrArray *radArray;
};


RadicalArray* radicalArray_new();

RadicalArray* radicalArray_sized_new(guint reserved_size);

RadicalType radicalArray_get_radicalType(RadicalArray *self);

void radicalArray_set_radicalType(RadicalArray *self,RadicalType type);

gpointer radicalArray_free(RadicalArray *self, gboolean free_segment);

void radicalArray_clone(RadicalArray *self);

void radicalArray_copy(RadicalArray *self,RadicalArray *src);

void radicalArray_reset(RadicalArray *self);

guint radicalArray_size(RadicalArray *self);

void radicalArray_add(RadicalArray *self,gpointer obj);

gboolean radicalArray_has_radicalCode(RadicalArray *self,RadicalCode code);

gpointer radicalArray_index(RadicalArray *self,gint index);

gpointer radicalArray_find_by_code(RadicalArray *self,RadicalCode code);

gboolean radicalArray_is_empty(RadicalArray *self);



#endif  /* RADICAL_ARRAY_H_ */

