/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */
#include <unistd.h>
#include "WritRecogn.h"
#include "algors.h"
#include "fileProcessing.h"


void init(int argc, char **argv) {
    progAssoc.progCmd=CHARACTER_DATA_MANAGER ;
    common_init(argc,argv);
    progAssoc.choose_open_callback=filename_get_user_chosen_callback_open_TUI;
    progAssoc.choose_save_callback=filename_get_user_chosen_callback_save_TUI;
    progAssoc.training_progress_callback=strokeHypothesisFile_start_training_TUI;
}


int main(int argc, char **argv) {
    int i,len;
    init(argc,argv);

//    RawCharacterList *rCharList=NULL;
//    WubiRecList *wubiList=NULL;


    RadicalList *radicalList;
    
    if (tomoeFileName) {
//        rCharList=tomoeCharacterDataFile_read_rawCharacterList(tomoeFileName);
//        LangAppearedSet langSet;
//        if (langStr!=NULL) {
//            langAppearedSet_set(&langSet,langStr,TRUE);
//        } else {
//            langAppearedSet_set(&langSet,"",TRUE);
//        }
//        rawCharacterDataFile_set_defaultLangs(&langSet);
//        rawCharacterDataFile_write_rawCharacterList(progAssoc.rawCharacterDataFileName,rCharList);
    } else if (wubiFileName){
//        rCharList=rawCharacterDataFile_read_rawCharacterList(progAssoc.rawCharacterDataFileName);
//        wubiList=wubiDataFile_read_wubiRecList(wubiFileName);
//        wubiRecList_applyOn_rawCharacterList(wubiList,rCharList);
//        rawCharacterDataFile_write_rawCharacterList(progAssoc.rawCharacterDataFileName,rCharList);
    } else if (destFileName){
	/* Export to other format */
	CharacterDataFile *destDataFile=writrecogn_character_datafile_new_from_file
	    (destFileName,FILE_OPEN_TYPE_AUTO);
	writrecogn_character_datafile_open(destDataFile);
	if (writrecogn_character_datafile_get_openType(destDataFile)==FILE_OPEN_TYPE_READ_ONLY){
	    printf("File %s is read-only!\n",progAssoc.strokeHypothesisFileName);
	    exit(1);
	}
	progAssoc.characterDataFile=writrecogn_character_datafile_new_from_file
	    (progAssoc.characterDataFileName,FILE_OPEN_TYPE_READ_ONLY);
	writrecogn_character_datafile_open(progAssoc.characterDataFile);
	progAssoc.currRadicalList=
	    writrecogn_character_datafile_read_radicalList(progAssoc.characterDataFile);

	writrecogn_character_datafile_append_radicalList(destDataFile,progAssoc.currRadicalList);

	writrecogn_character_datafile_close(destDataFile,TRUE);
	writrecogn_character_datafile_close(progAssoc.characterDataFile,TRUE);

    } else if (!isEmptyString(progAssoc.strokeHypothesisFileName)){
	/* Training or Testing the stroke hypothesis */
	progAssoc.characterDataFile=writrecogn_character_datafile_new_from_file(
		progAssoc.characterDataFileName,FILE_OPEN_TYPE_READ_ONLY);
	writrecogn_character_datafile_open(progAssoc.characterDataFile);
	progAssoc.currRadicalList
	    =writrecogn_character_datafile_read_radicalList(progAssoc.characterDataFile);
	if (progAssoc.flags.trainingFlag){
	    /* Training */
	    TrainingProgressData *tProgress=trainingProgressData_new();
	    writrecogn_stroke_recognizer_load_hypothesis(progAssoc.recognizer,"data/WritRecogn.svm.hypo");
	    tProgress->hypo=writrecogn_stroke_recognizer_get_hypothesis(progAssoc.recognizer);
	    writrecogn_stroke_recognizer_set_hypothesis(progAssoc.recognizer,tProgress->hypo);
	    gboolean ret=writrecogn_stroke_recognizer_save_hypothesis(progAssoc.recognizer,
		    progAssoc.strokeHypothesisFileName);
	    trainingProgressData_free(tProgress);
	    writrecogn_character_datafile_close(progAssoc.characterDataFile,TRUE);
	    if (ret){
		printf("svm model saved to %s\n",progAssoc.strokeHypothesisFileName);
	    }else{
		printf("Failed to save svm model in file %s\n",progAssoc.strokeHypothesisFileName);
		exit(1);
	    }
	}else{
	    /* Testing */
	    printf("Sorry, test for %s is not implemented yet!\n",CHARACTER_DATA_MANAGER_PROG_NAME);
	}
    } else if (!isEmptyString(inputCode)){

	progAssoc.characterDataFile=writrecogn_character_datafile_new_from_file(
		progAssoc.characterDataFileName,FILE_OPEN_TYPE_READ_ONLY);
	writrecogn_character_datafile_open(progAssoc.characterDataFile);

	progAssoc.currRadicalList=
	    writrecogn_character_datafile_read_radicalList(progAssoc.characterDataFile);
	RadicalQuery *rQuery=radicalQuery_new();
	radicalQuery_add_query(rQuery,RADICAL_FIELD_INPUT_CODE,QUERY_RELATION_EQUAL,inputCode);
	radicalList=writrecogn_character_datafile_find_matches(progAssoc.characterDataFile,rQuery);
	len=writrecogn_radical_list_size(radicalList);
        for(i=0;i<len;i++){
	    FullCharacter *fChar=FULLCHARACTER(writrecogn_radical_list_get_radical_by_index(radicalList,i,RADICAL_TYPE_FULLCHARACTER));
	    printf("Result \t%d\n",i);
	    printf("%s\n",writrecogn_radical_to_string(RADICAL(fChar)));
	}
	writrecogn_character_datafile_close(progAssoc.characterDataFile,TRUE);
    } else {
        progAssoc.characterDataFile=writrecogn_character_datafile_new_from_file(
		progAssoc.characterDataFileName,FILE_OPEN_TYPE_READ_ONLY);
	writrecogn_character_datafile_open(progAssoc.characterDataFile);

	radicalList=progAssoc.currRadicalList=
	    writrecogn_character_datafile_read_radicalList(progAssoc.characterDataFile);
	if (queryWord!=NULL) {
            FullCharacter* fChar=FULLCHARACTER(writrecogn_radical_list_find_radical_by_utf8(radicalList,
		    queryWord,RADICAL_TYPE_FULLCHARACTER));
            if (fChar==NULL) {
                printf("\nCharacter %s is not found.\n",queryWord);
            } else {
                printf("%s\n",writrecogn_radical_to_string(RADICAL(fChar)));
                // RawCharacter* rChar2=g_object_new(RAW_CHARACTER_TYPE,NULL);
                // rawCharacter_copy(rChar2,rChar);
                // printf("RChar2 %s\n",rawCharacter_to_string(rChar));
            }
        }
	writrecogn_character_datafile_close(progAssoc.characterDataFile,TRUE);
    }

    /*
    * this is to debug memory for regression tests
    */
    xmlMemoryDump();
    return(0);
}

