/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *  
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc. 
 * 
 * Red Hat Author(s): Ding-Yi Chen <dchen at redhat.com> 
 */

#include <stdio.h>
#include <libgen.h>
#include <unistd.h>
#include "common.h"


ProgramAssociate progAssoc;
gchar *queryWord=NULL;
gchar *langStr=NULL;
gchar *cdFileName=NULL;
gchar *shFileName=NULL;
InputMethod iMethod;
gchar *inputCode=NULL;

void printUsage(){
    puts(progAssoc.usageMsg);
}


/**
 * Whether the command line options are valid.
 */
gboolean is_valid_arguments(int argc, char **argv) {
    int opt;
    initString(progAssoc.characterDataFileName);
    initString(progAssoc.strokeHypothesisFileName);
    progAssoc.displayMode=RECOGNIZING_MODE;

    while ((opt = getopt(argc, argv, "hDETtC:H:I:L:M:R:Q:W:")) != -1) {
        switch (opt) {
            case 'h':
                printUsage();
                break;
	    case 'C':
		queryWord=optarg;
		// Query a character, implies TRAINING_MODE.
		progAssoc.displayMode=TRAINING_MODE;
		break;
            case 'D':
		setVerboseLevel(VERBOSE_MSG_INFO1);
                break;
            case 'E':
                progAssoc.displayMode=TRAINING_MODE;
                break;
	    case 'H':
		if (progAssoc.progCmd==CHARACTER_DATA_MANAGER){
		    shFileName=optarg;
		}
		break;
	    case 'I':
                if (progAssoc.progCmd==CHARACTER_DATA_MANAGER){
                    iMethod=inputMethod_parse(optarg);
                }
                break;
            case 'L':
                if (progAssoc.progCmd==CHARACTER_DATA_MANAGER){
                    langStr=optarg;
                }
                break;
	    case 'M':
		if (progAssoc.progCmd==CHARACTER_DATA_MANAGER){
		    tomoeFileName=optarg;
		}
		break;
	    case 'Q':
                if (progAssoc.progCmd==CHARACTER_DATA_MANAGER){
                    destFileName=optarg;
                }
                break;
            case 'R':
                if (progAssoc.progCmd==CHARACTER_DATA_MANAGER){
                    inputCode=optarg;
                }
                break;
	    case 'T':
		if (progAssoc.progCmd==CHARACTER_DATA_MANAGER){
		    progAssoc.flags.trainingFlag=TRUE;
		}
		break;
	    case 't':
		if (progAssoc.progCmd==CHARACTER_DATA_MANAGER){
		    progAssoc.flags.trainingFlag=FALSE;
		}
		break;
	    case 'W':
                if (progAssoc.progCmd==CHARACTER_DATA_MANAGER){
                    wubiFileName=optarg;
                }
                break;
            default: /* ’?’ */
                printf("Unrecognized Option -%c\n",opt);
                return FALSE;
        }
    }

    cdFileName=argv[optind];
    return TRUE;
}


/**
 * Check the availability of files.
 * 
 * If files are undefined, then use the default filenames
 */
gboolean check_files(){
    gchar uBuf[PATH_MAX];
    gchar sBuf[PATH_MAX];
    gchar cmd[PATH_MAX];
    truepath(DEFAULT_USER_DATA_DIR,uBuf);
    truepath(DEFAULT_SYSTEM_DATA_DIR,sBuf);

    if (progAssoc.progCmd==RECOGNIZER){
	if (access(uBuf,F_OK)!=0){
	    if (access(sBuf,F_OK)==0){
		printf("%s do not exist, copying from %s.\n ",uBuf,sBuf);
		sprintf(cmd,"cp -R %s %s",sBuf,uBuf);
		int ret=system(cmd);
		if (ret==0){
		    printf("Copy completed.\n");
		}else{
		    printf("Copy failed.\n");
		    exit(ret);
		}
	    }else{
		printf("%s or %s do not exist!",uBuf, sBuf);
		exit(1);
	    }
	}
    }

    /* 
     * For stroke data manager, if rawCharacterDataFile is not defined, then report error
     *
     * For recognizer, if rawCharacterDataFile is not defined, then used the default one 
     */
    if (cdFileName==NULL){
        if ( progAssoc.progCmd==CHARACTER_DATA_MANAGER){
            fprintf(stderr,"Please specify stroke data file\n");
            return FALSE;
        }else{
	    cdFileName=characterDataFile_get_preferredUserFileName();
        }
    }
    truepath(cdFileName,progAssoc.characterDataFileName);
    if (!isReadable(progAssoc.characterDataFileName)){
        fprintf(stderr,"Cannot read %s\n",progAssoc.characterDataFileName);
        initString(progAssoc.characterDataFileName);
        return FALSE;
    }

    /* 
     * For stroke data manager, if strokeHypothesisFile is not defined, then skip 
     *
     * For recognizer, if strokeHypothesisFile is not defined, then used the default one,
     *  if default one does not exist (not R_OK), then skip.
     */
    if (progAssoc.progCmd==RECOGNIZER){
        if (shFileName==NULL){
            shFileName=strokeHypothesisFile_get_preferredUserFileName();
        }
        truepath(shFileName,progAssoc.strokeHypothesisFileName);
        if (!isReadable(progAssoc.strokeHypothesisFileName)){
            fprintf(stderr,"Cannot read the stroke hypothesisfile %s, skip",progAssoc.strokeHypothesisFileName);
            initString(progAssoc.strokeHypothesisFileName);
            shFileName=NULL;
            // Skip
        }
    }else{
        if (shFileName){
	    if (progAssoc.flags.trainingFlag){
		if (isWritable(shFileName)){
		    truepath(shFileName,progAssoc.strokeHypothesisFileName);
		}else{
		    fprintf(stderr,"Cannot write the stroke hypothesis file %s",shFileName);
		    shFileName=NULL;
		    return FALSE;
		}
	    }else{
		if (isReadable(shFileName)){
		    truepath(shFileName,progAssoc.strokeHypothesisFileName);
		}else{
		    fprintf(stderr,"Cannot read the stroke hypothesis file %s",shFileName);
		    shFileName=NULL;
		    return FALSE;
		}
	    }
        }
    }
    return TRUE;
}

void
common_init(int argc,char **argv){
    setVerboseLevel(VERBOSE_MSG_WARNING);
       
    progAssoc.flags.initializedFlag=FALSE;
    progAssoc.flags.guiFlag=(progAssoc.progCmd==RECOGNIZER)? TRUE: FALSE;
    progAssoc.flags.trainingFlag=FALSE;
    progAssoc.flags.canvasModified=FALSE;
    progAssoc.flags.characterDataModified=FALSE;
    progAssoc.flags.hypothesisModified=FALSE;
    progAssoc.displayMode=RECOGNIZING_MODE;
    progAssoc.usageMsg=(progAssoc.progCmd==RECOGNIZER)? RECOGNIZER_USAGE:  CDMGR_USAGE;
    
    
    if (!is_valid_arguments(argc,argv)) {
        printf(" Arguments invalid !\n");
        printUsage();
        exit(1);
    }
    
    if (!check_files()){
        printf(" Files Error !\n");
        printUsage();
        exit(1);
    }

    g_type_init();
    progAssoc.charMatcher=writrecogn_character_matcher_naive_new();
    progAssoc.strokeNoiseReducer=writrecogn_stroke_noise_reducer_find_farthest_nodes_new();
    progAssoc.recognizer=writrecogn_stroke_recognizer_libsvm_new();
}

