/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

/**
 * Common data structures and functions for both GUI and TUI
 */

#ifndef COMMON_H_
#define COMMON_H_
#include <glib.h>
//#include <sqlite3.h>
//#include "characterData-SQLite.h"
//#include "characterData.h"
#include "WritRecogn.h"
#include "fileProcessing.h"
#include "algors.h"

#define MAIN_PROG_NAME "WritRecogn"
#define CHARACTER_DATA_MANAGER_PROG_NAME MAIN_PROG_NAME "-manager"
#define DEFAULT_CHARACTER_DATA_FILENAME MAIN_PROG_NAME 

#ifndef datadir
#define datadir "/usr/share"
#endif
#define DEFAULT_SYSTEM_DATA_DIR datadir "/" MAIN_PROG_NAME "/data/"
#define DEFAULT_USER_DATA_DIR "~/." MAIN_PROG_NAME "/"

#define DEFAULT_USER_CHARACTER_DATA_PATH_XML DEFAULT_USER_DATA_DIR DEFAULT_CHARACTER_DATA_FILENAME ".xml"
#define DEFAULT_USER_CHARACTER_DATA_PATH_SQLITE DEFAULT_USER_DATA_DIR DEFAULT_CHARACTER_DATA_FILENAME ".db"
#define DEFAULT_USER_CHARACTER_DATA_PATH DEFAULT_USER_CHARACTER_DATA_PATH_SQLITE

#define DEFAULT_STROKE_HYPOTHESIS_FILENAME MAIN_PROG_NAME ".svm.hypo"
#define DEFAULT_USER_STROKE_HYPOTHESIS_PATH  DEFAULT_USER_DATA_DIR DEFAULT_STROKE_HYPOTHESIS_FILENAME

#define PIXMAP_DIR datadir "/" MAIN_PROG_NAME "/pixmaps/" 

#define DEFAULT_WHOLE_RADICAL_DATA_SIZE 7000
#define MAX_STRING_LENGTH 200


#define COMMON_CMD_OPTIONS \
" -h: Display Usage\n"\
" -D: Print debug messages\n"\
" -C: specify character\n"

#define MAIN_PROG_CMD_OPTIONS \
" -E: Enter editing mode.\n"


#define CDMGR_CMD_OPTIONS \
" -L lang: set the language as lang"\
" -M tomoeStrokeXmlFile: Set tomoe stroke XML to be imported from.\n"\
" -R inputCode: Input code for a input method.\n"\
" -I inputMethod: Input method that generate the inputCode.\n"\
" -H <hypothesisFile>: Hypothesis file derived from Training character data\n"\
" -T: Train from character data\n"\
" -t: Test the correctness character [TODO] \n"\
" -W: Add Wubi CSV stroke data.\n"\
" -Q: Convert between XML and SQLite.\n"

#define RECOGNIZER_USAGE \
"\n " MAIN_PROG_NAME ": Character/Word Recognizer\n"\
"Usage: " MAIN_PROG_NAME " -[DEh] [strokeDataFile [char]]  \n"\
COMMON_CMD_OPTIONS \
MAIN_PROG_CMD_OPTIONS

#define CDMGR_USAGE \
"\n " CHARACTER_DATA_MANAGER_PROG_NAME ": Stroke data manager\n"\
"Usage:\n"\
" Query a char:\n"\
"\t" CHARACTER_DATA_MANAGER_PROG_NAME " [-D] -C char <CharacterDataFile>\n"\
" Export to SQLite (if CharacterData_dest has .db postfix) \n"\
"        or XML (if CharacterData_dest has .xml postfix):\n"\
"\t" CHARACTER_DATA_MANAGER_PROG_NAME " [-D] -Q <CharacterData_dest> <CharacterDataFile>  \n"\
" Given input code, find the corresponding char:\n"\
"\t" CHARACTER_DATA_MANAGER_PROG_NAME " [-D] -R <inputCode> [-I inputMethod] <CharacterDataFile> \n"\
" Import from Tomoe:\n"\
"\t" CHARACTER_DATA_MANAGER_PROG_NAME " [-D] [-L lang] -M <tomoeStrokeXmlFile> <CharacterDataFile>\n"\
" Train from the character data:\n"\
"\t" CHARACTER_DATA_MANAGER_PROG_NAME " [-D] -T -H <hypothesisFile> <CharacterDataFile>\n"\
" Test from the character data [TODO]:\n"\
"\t" CHARACTER_DATA_MANAGER_PROG_NAME " [-D] -t -H <hypothesisFile> <CharacterDataFile>\n"\
" Add Wubi CSV stroke data:\n"\
"\t" CHARACTER_DATA_MANAGER_PROG_NAME " [-D] [-L lang] -W <wubiCSVfile> <CharacterDataFile>\n"\
"Options:\n"\
COMMON_CMD_OPTIONS \
CDMGR_CMD_OPTIONS

/**
 * Parameters and global variables of Program
 *
 */
struct _ProgramAssociate {
    MainProgramCmd progCmd;
    ProgFlags flags;
    gchar *usageMsg;

    RadicalList *currRadicalList;
    CharacterMatcher *charMatcher;
    StrokeRecognizer *recognizer;
    StrokeNoiseReducer *strokeNoiseReducer;

    CharacterDataFile *characterDataFile;
    gchar characterDataFileName[PATH_MAX];
    gchar strokeHypothesisFileName[PATH_MAX]; // Decision rules of stroke recognition
    DisplayMode displayMode;


    /**
     * choose_open_callback:
     * Callback function for choose file to open.
     * 
     */
    ChooseFilenameFunc choose_open_callback;

    /**
     * choose_save_callback:
     * Callback function for choose file to save.
     * 
     */
    ChooseFilenameFunc choose_save_callback;

    /**
     * training_progress_callback:
     * Callback function for knowing current progress of training.
     * 
     */
    TrainingProgressFunc training_progress_callback;
};

/**
 * Parameters from command line options
 */
extern ProgramAssociate progAssoc;
extern gchar *queryWord;
extern gchar *langStr;
extern gchar *inputCode;
extern InputMethod iMethod;

void
common_init(int argc,char **argv);

#endif /*COMMON_H_*/
