/*
 * Copyright © 2008  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */
#include <glib.h>
#include <sqlite3.h>
#include "str_functions.h"
#include "characterData-SQLite-table.h"

WritRecognDB_properties writRecognDB_properties;

unsigned tableFlagValues[] = {
    1 << RELATIVEBOUNDINGBOX_TABLE_INDEX,
    1 << RELATIVERADICALSEQUENCE_TABLE_INDEX,
    1 << LANG_TABLE_INDEX,
    1 << VARIANT_TABLE_INDEX,
    1 << INPUTCODE_TABLE_INDEX,
    1 << STROKEDATA_TABLE_INDEX,
    1 << WRITRECOGN_TABLE_INDEX,
};

FieldProperties relativeBoundingBox_fields[] = RELATIVEBOUNDINGBOX_TABLE_FIELDS;
FieldProperties relativeRadicalSequence_fields[] = RELATIVERADICALSEQUENCE_TABLE_FIELDS;
FieldProperties strokeDataTable_fields[] = STROKEDATA_TABLE_FIELDS;
FieldProperties langTable_fields[] = LANG_TABLE_FIELDS;
FieldProperties variantTable_fields[] = VARIANT_TABLE_FIELDS;
FieldProperties inputCodeTable_fields[] = INPUTCODE_TABLE_FIELDS;
FieldProperties writRecognTable_fields[] = WRITRECOGN_TABLE_FIELDS;

int radicalArray_parse_result_callback_relativeBoundingBoxTable
(void *radicalList, int colNum, char **recStr, char **azColName);
gchar *radical_to_insertCmds_relativeBoundingBoxTable(gpointer obj);
gchar *radical_to_updateCmds_relativeBoundingBoxTable(gpointer obj);

int radicalArray_parse_result_callback_relativeRadicalSequenceTable
(void *radicalList, int colNum, char **recStr, char **azColName);
gchar *radical_to_insertCmds_relativeRadicalSequenceTable(gpointer obj);
gchar *radical_to_updateCmds_relativeRadicalSequenceTable(gpointer obj);

int radicalArray_parse_result_callback_strokeDataTable
(void *radicalList, int colNum, char **recStr, char **azColName);
gchar *radical_to_insertCmds_strokeDataTable(gpointer obj);
gchar *radical_to_updateCmds_strokeDataTable(gpointer obj);

int radicalArray_parse_result_callback_langTable
(void *radicalList, int colNum, char **recStr, char **azColName);
gchar *radical_to_insertCmds_langTable(gpointer obj);
gchar *radical_to_updateCmds_langTable(gpointer obj);

int radicalArray_parse_result_callback_variantTable
(void *radicalList, int colNum, char **recStr, char **azColName);
gchar *radical_to_insertCmds_variantTable(gpointer obj);
gchar *radical_to_updateCmds_variantTable(gpointer obj);

int radicalArray_parse_result_callback_inputCodeTable
(void *radicalList, int colNum, char **recStr, char **azColName);
gchar *radical_to_insertCmds_inputCodeTable(gpointer obj);
gchar *radical_to_updateCmds_inputCodeTable(gpointer obj);


TableProperties dataTables[] = {
    {"RelativeBoundingBoxTable", relativeBoundingBox_fields,
	radicalArray_parse_result_callback_relativeBoundingBoxTable,
	radical_to_insertCmds_relativeBoundingBoxTable,
	radical_to_updateCmds_relativeBoundingBoxTable},
    {"RelativeRadicalSequenceTable", relativeRadicalSequence_fields,
	radicalArray_parse_result_callback_relativeRadicalSequenceTable,
	radical_to_insertCmds_relativeRadicalSequenceTable,
	radical_to_updateCmds_relativeRadicalSequenceTable},    
    {"LangTable", langTable_fields,
	radicalArray_parse_result_callback_langTable,
	radical_to_insertCmds_langTable,
	radical_to_updateCmds_langTable},
    {"VariantTable", variantTable_fields,
	radicalArray_parse_result_callback_variantTable,
	radical_to_insertCmds_variantTable,
	radical_to_updateCmds_variantTable},
    {"InputCodeTable", inputCodeTable_fields,
	radicalArray_parse_result_callback_inputCodeTable,
	radical_to_insertCmds_inputCodeTable,
	radical_to_updateCmds_inputCodeTable},
    {"StrokeDataTable", strokeDataTable_fields,
	radicalArray_parse_result_callback_strokeDataTable,
	radical_to_insertCmds_strokeDataTable,
	radical_to_updateCmds_strokeDataTable},
    {"WritRecognTable", writRecognTable_fields,
	writRecognDB_properties_parse_result_callback,
	writRecognDB_properties_to_insertCmds,
	writRecognDB_properties_to_updateCmds	},
    NULL
};

/*================ RadicalList Functions ====================*/

int radicalList_parse_result_callback
(void *radicalList, int colNum, char **recStr, char **azColName)
{
    RadicalList* radList=(RadicalList *) radicalList;
    RadicalCode currRadicalCode = atol(recStr[0]);
    if (!writrecogn_radical_list_has_radicalCode(radList,currRadicalCode)){
	writrecogn_radical_list_append_radicalCode(radList,currRadicalCode);
    }
    return 0;
}

/*================ RelativeBoundingBoxTable Functions ====================*/
int radicalArray_parse_result_callback_relativeBoundingBoxTable
(void *radicalArray, int colNum, char **recStr, char **azColName){
    int i;
    static RadicalCode lastRadicalCode = 0;
    RadicalCode currRadicalCode;

    static gint lastBoxSequenceId = 0;
    gint currBoxSequenceId = 0;

    static gint lastSubRadicalId = 0;
    gint currSubRadicalId = 0;

    static RadicalCode lastSubRadicalCode = 0;
    RadicalCode currSubRadicalCode;

    gint left,top,right,bottom;
    RawWriting *rW=NULL;
    static RawStroke *rStroke = NULL;
    FullCharacter *fChar;
    Radical *subRad;
    RadicalArray *radArray = (RadicalArray *) radicalArray;
    BoundingBox box;

    for (i = 0; i < colNum; i++) {
	switch (i) {
	    case RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_RADICALCODE:
		currRadicalCode = atol(recStr[i]);
		fChar=radicalArray_find_by_code(radArray,currRadicalCode);
		if (!fChar){
		    fChar=writrecogn_fullcharacter_new();
		    writrecogn_radical_set_radicalCode(RADICAL(fChar),currRadicalCode);
		    radicalArray_add(radArray,fChar);
		}
		if (lastRadicalCode != currRadicalCode) {
		    lastRadicalCode = currRadicalCode;
		    lastBoxSequenceId = 0;
		    lastSubRadicalId = 0;
		    lastSubRadicalCode = 0;
		}
		break;
	    case RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_BOXSEQUENCEID:
		currBoxSequenceId = atol(recStr[i]);
		if (lastBoxSequenceId != currBoxSequenceId) {
		    lastBoxSequenceId = currBoxSequenceId; 
		    lastSubRadicalId = 0;
		    lastSubRadicalCode = 0;
		}
		break;
	    case RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_SUBRADICALID:
		currSubRadicalId = atol(recStr[i]);
		if (lastSubRadicalId != currSubRadicalId) {
		    lastSubRadicalId = currSubRadicalId;
		    lastSubRadicalCode = 0;
		}
		break;
	    case RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_SUBRADICALCODE:
		currSubRadicalCode = atol(recStr[i]);
		subRad=writrecogn_radical_new();
		writrecogn_radical_set_radicalCode(subRad,currSubRadicalCode);
		if (lastSubRadicalCode != currSubRadicalCode) {
		    lastSubRadicalCode = currSubRadicalCode;
		}
		break;
	    case RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_LEFT:
		box.left=left = atol(recStr[i]);
		break;
	    case RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_TOP:
		box.top=top = atol(recStr[i]);
		break;
	    case RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_RIGHT:
		box.right=right = atol(recStr[i]);
		break;
	    case RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_BOTTOM:
		box.bottom=bottom = atol(recStr[i]);
		writrecogn_radical_set_relativeBoundingBox(subRad,&box);
		writrecogn_radical_add_subRadical(RADICAL(fChar),subRad);
		break;

	}
    }
    return 0;
}

gchar *radical_to_insertCmds_relativeBoundingBoxTable(gpointer obj)
{
    Radical *rad=(Radical *) obj;
    GString *sBuf = g_string_new("");
    int i,len=writrecogn_radical_count_subRadicals(rad);
    Radical *subRad;
    BoundingBox *box;
    for (i = 0; i < len; i++) {
	subRad=writrecogn_radical_get_subRadical(rad,i);
	box=writrecogn_radical_get_relativeBoundingBox(subRad);
	g_string_append_printf(sBuf, "INSERT INTO %s VALUES (", 
		dataTables[RELATIVEBOUNDINGBOX_TABLE_INDEX].name);
	g_string_append_printf(sBuf, "%d, %u, %u, %d, %u, %u, %u, %u);\n",
		writrecogn_radical_get_radicalCode(rad), 1, i+1, 
		writrecogn_radical_get_radicalCode(subRad), 
		box->left, box->top, box->right, box->bottom);
    }
    return g_string_free(sBuf, FALSE);
}

gchar *radical_to_updateCmds_relativeBoundingBoxTable(gpointer obj)
{
    Radical *rad=(Radical *) obj;
    GString *sBuf = g_string_new("");
    g_string_append_printf(sBuf, "DELETE FROM %s WHERE ", dataTables[RELATIVEBOUNDINGBOX_TABLE_INDEX].name);
    g_string_append_printf(sBuf, "%s = %d ;\n", MAIN_KEY_FIELD,
	    writrecogn_radical_get_radicalCode(rad));
    g_string_append_printf(sBuf, "%s", radical_to_insertCmds_relativeBoundingBoxTable(rad));
    return g_string_free(sBuf, FALSE);
}

/*================ RelativeRadicalSequenceTable Functions ====================*/
int radicalArray_parse_result_callback_relativeRadicalSequenceTable
(void *radicalArray, int colNum, char **recStr, char **azColName){
    int i;
    static RadicalCode lastRadicalCode = 0;
    RadicalCode currRadicalCode;

    FullCharacter *fChar;
    Radical *subRad;
    RadicalArray *radArray = (RadicalArray *) radicalArray;

    for (i = 0; i < colNum; i++) {
	switch (i) {
	    case RELATIVERADICALSEQUENCE_TABLE_FIELD_ID_RADICALCODE:
		currRadicalCode = atol(recStr[i]);
		fChar=radicalArray_find_by_code(radArray,currRadicalCode);
		if (!fChar){
		    fChar=writrecogn_fullcharacter_new();
		    writrecogn_radical_set_radicalCode(RADICAL(fChar),currRadicalCode);
		    radicalArray_add(radArray,fChar);
		}
		if (lastRadicalCode != currRadicalCode) {
		    lastRadicalCode = currRadicalCode;
		}
		break;
	    default:
		/* 
		 * The other fields are not required for AbsCharacter construction,
		 * because the absCharacter->subRadical_sequence_H and 
		 * absCharacter->subRadical_sequence_H are built after called
		 * writrecogn_radical_add_subRadical() from
		 * radicalArray_parse_result_callback_relativeBoundingBoxTable()
		 */
		break;

	}
    }
    return 0;
}

gchar *radical_to_insertCmds_relativeRadicalSequenceTable(gpointer obj)
{
    Radical *rad=(Radical *) obj;
    AbsCharacter* aChar=ABSCHARACTER(rad);
    GString *sBuf = g_string_new(NULL);
    GPtrArray *seqH=writrecogn_abscharacter_get_subRadical_sequence_H(aChar);
    GPtrArray *seqV=writrecogn_abscharacter_get_subRadical_sequence_V(aChar);
    int i,hLen=seqH->len,vLen=seqV->len;
    g_assert(hLen==vLen);
    Radical *subRad;
    g_string_append_printf(sBuf, "INSERT INTO %s VALUES ( %d, '", 
	    dataTables[RELATIVERADICALSEQUENCE_TABLE_INDEX].name,
	    writrecogn_radical_get_radicalCode(rad) );

    for (i = 0; i < hLen; i++) {
	subRad=g_ptr_array_index(seqH,i);
	if (i>0)
	    g_string_append(sBuf," ");
	g_string_append_printf(sBuf, "%d",writrecogn_radical_get_radicalCode(subRad));
    }
    g_string_append(sBuf, "', '");
    for (i = 0; i < vLen; i++) {
	subRad=g_ptr_array_index(seqV,i);
	if (i>0)
	    g_string_append(sBuf," ");
	g_string_append_printf(sBuf, "%d",writrecogn_radical_get_radicalCode(subRad));
    }
    g_string_append(sBuf, "') ;");
    return g_string_free(sBuf, FALSE);
}

gchar *radical_to_updateCmds_relativeRadicalSequenceTable(gpointer obj)
{
    Radical *rad=(Radical *) obj;
    GString *sBuf = g_string_new("");
    g_string_append_printf(sBuf, "DELETE FROM %s WHERE ", dataTables[RELATIVERADICALSEQUENCE_TABLE_INDEX].name);
    g_string_append_printf(sBuf, "%s = %d ;\n", MAIN_KEY_FIELD,
	    writrecogn_radical_get_radicalCode(rad));
    g_string_append_printf(sBuf, "%s", radical_to_insertCmds_relativeRadicalSequenceTable(rad));
    return g_string_free(sBuf, FALSE);
}

/*================ LangTable Functions ====================*/
int radicalArray_parse_result_callback_langTable(void *radicalArray, int colNum, char **recStr, char **azColName)
{
    int i;
    static RadicalCode lastRadicalCode = 0;
    RadicalCode currRadicalCode;
    RadicalArray *radArray = (RadicalArray *) radicalArray;
    FullCharacter *fChar;

    for (i = 0; i < colNum; i++) {
        switch (i) {
        case LANG_TABLE_FIELD_ID_RADICALCODE:
	    currRadicalCode = atol(recStr[i]);
	    fChar=radicalArray_find_by_code(radArray,currRadicalCode);
	    if (!fChar){
		fChar=writrecogn_fullcharacter_new();
		writrecogn_radical_set_radicalCode(RADICAL(fChar),currRadicalCode);
		radicalArray_add(radArray,fChar);
	    }
	    if (lastRadicalCode != currRadicalCode) {
		lastRadicalCode = currRadicalCode;
	    }
	    break;
        case LANG_TABLE_FIELD_ID_LANG:
	    writrecogn_abscharacter_add_language_string(ABSCHARACTER(fChar),recStr[i]);
            if (lastRadicalCode != currRadicalCode) {
		verboseMsgPrint(FILE_VERBOSE_LEVEL, "Character %s\n", 
			writrecogn_radical_to_string(RADICAL(fChar)));
                lastRadicalCode = currRadicalCode;
            }
            break;
        }
    }
    return 0;
}


gchar *radical_to_insertCmds_langTable(gpointer obj)
{
    Radical *rad=(Radical *) obj;
    GString *sBuf = g_string_new("");
    gint i, j;
    AbsCharacter *aChar=ABSCHARACTER(rad);
    if (writrecogn_abscharacter_has_language(aChar,LANG_ZH_SIMPLIFIED)) {
        g_string_append_printf(sBuf, "INSERT INTO %s VALUES (", dataTables[LANG_TABLE_INDEX].name);
        g_string_append_printf(sBuf, "%d, '%s');\n", rad->radicalCode,  ZH_SIMPLIFIED_STR);
    }
    if (writrecogn_abscharacter_has_language(aChar,LANG_ZH_TRADITIONAL)) {
        g_string_append_printf(sBuf, "INSERT INTO %s VALUES (", dataTables[LANG_TABLE_INDEX].name);
        g_string_append_printf(sBuf, "%d, '%s');\n", rad->radicalCode, ZH_TRADITIONAL_STR);
    }
    if (writrecogn_abscharacter_has_language(aChar,LANG_JP)) {
        g_string_append_printf(sBuf, "INSERT INTO %s VALUES (", dataTables[LANG_TABLE_INDEX].name);
        g_string_append_printf(sBuf, "%d, '%s');\n", rad->radicalCode, "jp");
    }
    if (writrecogn_abscharacter_has_language(aChar,LANG_KR)) {
        g_string_append_printf(sBuf, "INSERT INTO %s VALUES (", dataTables[LANG_TABLE_INDEX].name);
        g_string_append_printf(sBuf, "%d, '%s');\n", rad->radicalCode, "kr");
    }
    return g_string_free(sBuf, FALSE);
}


gchar *radical_to_updateCmds_langTable(gpointer obj)
{
    Radical *rad=(Radical *) obj;
    GString *sBuf = g_string_new("");
    g_string_append_printf(sBuf, "DELETE FROM %s WHERE ", dataTables[LANG_TABLE_INDEX].name);
    g_string_append_printf(sBuf, "%s = %d ;\n", MAIN_KEY_FIELD, rad->radicalCode);
    g_string_append_printf(sBuf, "%s", radical_to_insertCmds_langTable(rad));
    return g_string_free(sBuf, FALSE);
}


/*================ VariantTable Functions ====================*/

int radicalArray_parse_result_callback_variantTable(void *radicalArray, int colNum, char **recStr, char **azColName)
{
    int i;
    static RadicalCode lastRadicalCode = 0;
    RadicalCode currRadicalCode;
    RadicalCode currVarRadicalCode;

    Radical *rad = NULL;
    RadicalArray *radArray = (RadicalArray *) radicalArray;
    FullCharacter *fChar;

    
    for (i = 0; i < colNum; i++) {
        switch (i) {
        case VARIANT_TABLE_FIELD_ID_RADICALCODE:
	    currRadicalCode = atol(recStr[i]);
	    fChar=radicalArray_find_by_code(radArray,currRadicalCode);
	    if (!fChar){
		fChar=writrecogn_fullcharacter_new();
		writrecogn_radical_set_radicalCode(RADICAL(fChar),currRadicalCode);
		radicalArray_add(radArray,fChar);
	    }
	    if (lastRadicalCode != currRadicalCode) {
		lastRadicalCode = currRadicalCode;
	    }
            break;
        case VARIANT_TABLE_FIELD_ID_VARRADICALCODE:
            currVarRadicalCode =  atol(recStr[i]);
	    writrecogn_abscharacter_insert_variantCharacter_by_code(ABSCHARACTER(fChar),currVarRadicalCode);
            break;
        }
    }
    return 0;
}

gchar *radical_to_insertCmds_variantTable(gpointer obj)
{
    Radical *rad=(Radical *) obj;
    AbsCharacter *aChar=ABSCHARACTER(rad);
    GString *sBuf = g_string_new("");
    gint i, j;
    glong items_written;
    gint len = writrecogn_abscharacter_count_variantCharacters(aChar);

    for (i = 0; i < len; i++) {
        Radical *varRad = writrecogn_abscharacter_get_variantCharacter(aChar, i,RADICAL_TYPE_RADICAL);
        g_string_append_printf(sBuf, "INSERT INTO %s VALUES (", dataTables[VARIANT_TABLE_INDEX].name);
        g_string_append_printf(sBuf, "%d, %u);\n", rad->radicalCode, varRad->radicalCode);
    }
    return g_string_free(sBuf, FALSE);
}


gchar *radical_to_updateCmds_variantTable(gpointer obj)
{
    Radical *rad=(Radical *) obj;
    GString *sBuf = g_string_new("");
    g_string_append_printf(sBuf, "DELETE FROM %s WHERE ", dataTables[VARIANT_TABLE_INDEX].name);
    g_string_append_printf(sBuf, "%s = %d ;\n", MAIN_KEY_FIELD, rad->radicalCode);
    g_string_append_printf(sBuf, "%s", radical_to_insertCmds_variantTable(rad));
    return g_string_free(sBuf, FALSE);
}

/*================ InputCodeTable Functions ====================*/
int radicalArray_parse_result_callback_inputCodeTable
(void *radicalArray, int colNum, char **recStr, char **azColName)
{
    int i;
    static RadicalCode lastRadicalCode = 0;
    RadicalCode currRadicalCode;
    InputCodeRec iRec;
    iRec.inputCode = NULL;
    RadicalArray *radArray = (RadicalArray *) radicalArray;
    FullCharacter *fChar;

    for (i = 0; i < colNum; i++) {
        switch (i) {
        case INPUTCODE_TABLE_FIELD_ID_RADICALCODE:
	    currRadicalCode = atol(recStr[i]);
	    fChar=radicalArray_find_by_code(radArray,currRadicalCode);
	    if (!fChar){
		fChar=writrecogn_fullcharacter_new();
		writrecogn_radical_set_radicalCode(RADICAL(fChar),currRadicalCode);
		radicalArray_add(radArray,fChar);
	    }
	    if (lastRadicalCode != currRadicalCode) {
		lastRadicalCode = currRadicalCode;
	    }
            break;
        case INPUTCODE_TABLE_FIELD_ID_METHOD:
            iRec.method = inputMethod_parse(recStr[i]);
            break;
        case INPUTCODE_TABLE_FIELD_ID_INPUTCODE:
	    iRec.inputCode=recStr[i];
	    writrecogn_abscharacter_append_inputCodeRec(ABSCHARACTER(fChar),&iRec);
            break;
        }
    }
    return 0;
}

gchar *radical_to_insertCmds_inputCodeTable(gpointer obj)
{
    Radical *rad=(Radical *) obj;
    GString *sBuf = g_string_new("");
    gint i, j;
    glong items_written;
    AbsCharacter *aChar=ABSCHARACTER(rad);
    gint len = writrecogn_abscharacter_count_inputCodeRecs(aChar);


    for (i = 0; i < len; i++) {
	InputCodeRec *iRec = writrecogn_abscharacter_get_inputCodeRec(aChar, i);
        g_string_append_printf(sBuf, "INSERT INTO %s VALUES (", dataTables[INPUTCODE_TABLE_INDEX].name);
        g_string_append_printf(sBuf, "%d, '%s', '%s');\n", rad->radicalCode,
                               inputMethod_to_string(iRec->method), iRec->inputCode);
    }
    return g_string_free(sBuf, FALSE);
}

gchar *radical_to_updateCmds_inputCodeTable(gpointer obj)
{
    Radical *rad=(Radical *) obj;
    GString *sBuf = g_string_new("");
    g_string_append_printf(sBuf, "DELETE FROM %s WHERE ", dataTables[INPUTCODE_TABLE_INDEX].name);
    g_string_append_printf(sBuf, "%s = %d ;\n", MAIN_KEY_FIELD, rad->radicalCode);
    g_string_append_printf(sBuf, "%s", radical_to_insertCmds_inputCodeTable(rad));
    return g_string_free(sBuf, FALSE);
}

/*================ StrokeDataTable Functions ====================*/
    int radicalArray_parse_result_callback_strokeDataTable
(void *radicalArray, int colNum, char **recStr, char **azColName)
{
    int i;
    static RadicalCode lastRadicalCode = 0;
    RadicalCode currRadicalCode;

    static gint lastWritingId = 0;
    static gint lastStrokeId = 0;
    static gint lastNodeId = 0;
    gint currWritingId,currStrokeId, currNodeId;
    gint nodeX, nodeY;
    RawWriting *rW=NULL;
    static RawStroke *rStroke = NULL;
    FullCharacter *fChar;

    RadicalArray *radArray = (RadicalArray *) radicalArray;

    for (i = 0; i < colNum; i++) {
	switch (i) {
	    case STROKE_DATA_TABLE_FIELD_ID_RADICALCODE:
		currRadicalCode = atol(recStr[i]);
		fChar=radicalArray_find_by_code(radArray,currRadicalCode);
		if (!fChar){
		    fChar=writrecogn_fullcharacter_new();
		    writrecogn_radical_set_radicalCode(RADICAL(fChar),currRadicalCode);
		    radicalArray_add(radArray,fChar);
		}
		if (lastRadicalCode != currRadicalCode) {
		    lastRadicalCode = currRadicalCode;
		    lastWritingId= 0;
		    lastStrokeId = lastNodeId = 0;
		}
		break;
	    case STROKE_DATA_TABLE_FIELD_ID_WRITINGID:
		currWritingId = atoi(recStr[i]);
		if (currWritingId != lastWritingId){
		    rW=writrecogn_fullcharacter_new_rawWriting(fChar);
		    rW->writingId=currWritingId;
		    lastWritingId=currWritingId;
		    lastStrokeId = lastNodeId = 0;
		}
		break;
	    case STROKE_DATA_TABLE_FIELD_ID_STROKEID:
		currStrokeId = atoi(recStr[i]);
		if (currStrokeId != lastStrokeId) {
		    rStroke = writrecogn_rawstroke_new();
		    writrecogn_fullcharacter_add_rawStroke(fChar,currWritingId-1,rStroke);
		    lastStrokeId = currStrokeId;
		    lastNodeId = 0;
		}
		break;
	    case STROKE_DATA_TABLE_FIELD_ID_NODEID:
		currNodeId = atoi(recStr[STROKE_DATA_TABLE_FIELD_ID_NODEID]);
		nodeX = atoi(recStr[STROKE_DATA_TABLE_FIELD_ID_NODEX]);
		nodeY = atoi(recStr[STROKE_DATA_TABLE_FIELD_ID_NODEY]);
		writrecogn_rawstroke_add_rawStrokeNode(rStroke, nodeX, nodeY);
		break;
	    case STROKE_DATA_TABLE_FIELD_ID_NODEX:
		break;
	    case STROKE_DATA_TABLE_FIELD_ID_NODEY:
		break;
	}
    }
    return 0;
}

gchar *radical_to_insertCmds_strokeDataTable(gpointer obj)
{
    
    Radical *rad=(Radical *) obj;
    FullCharacter* fChar=FULLCHARACTER(rad);
    GString *sBuf = g_string_new("");
    guint i, j, k;
    guint rawWritingCount = writrecogn_fullcharacter_count_rawWritings(fChar);
    guint strokeCount;
    guint nodeCount;
    
    for (i = 0; i < rawWritingCount; i++) {
	RawWriting *rW= writrecogn_fullcharacter_get_rawWriting(fChar,i);
	strokeCount=writrecogn_fullcharacter_count_rawStrokes(fChar,rW->writingId-1);
	for( j = 0; j < strokeCount; j++){
	    RawStroke *rStroke = writrecogn_fullcharacter_get_rawStroke(fChar,rW->writingId-1,j);
	    nodeCount = writrecogn_rawstroke_count_rawStrokeNodes(rStroke);
	    for (k=0;k<nodeCount;k++){
		RawStrokeNode *rStrokeNode = writrecogn_rawstroke_get_rawStrokeNode(rStroke, k);
		g_string_append_printf(sBuf, "INSERT INTO %s VALUES (", dataTables[STROKEDATA_TABLE_INDEX].name);
		g_string_append_printf(sBuf, "%d, %u, %u, %u, %u, %u);\n",
			writrecogn_radical_get_radicalCode(RADICAL(fChar)),
			i + 1, j + 1, k+1, rStrokeNode->x, rStrokeNode->y);
	    }
	}
    }
    return g_string_free(sBuf, FALSE);
}

gchar *radical_to_updateCmds_strokeDataTable(gpointer obj)
{
    Radical *rad=(Radical *) obj;
    GString *sBuf = g_string_new("");
    g_string_append_printf(sBuf, "DELETE FROM %s WHERE ", dataTables[STROKEDATA_TABLE_INDEX].name);
    g_string_append_printf(sBuf, "%s = %d ;\n", MAIN_KEY_FIELD,
	    writrecogn_radical_get_radicalCode(rad));
    g_string_append_printf(sBuf, "%s", radical_to_insertCmds_strokeDataTable(rad));
    return g_string_free(sBuf, FALSE);
}

/*================ WritRecognTable  Functions ====================*/

int writRecognDB_properties_parse_result_callback
(void *db_properties, int colNum, char **recStr, char **azColName)
{
    int i;
    WritRecognDB_properties *db_prop=(WritRecognDB_properties *) db_properties;

    for (i = 0; i < colNum; i++) {
	switch (i) {
	    case WRITRECOGN_TABLE_FIELD_ID_VERSION:
		db_prop->version = g_strdup(recStr[i]);
		break;
	}
    }
    return 0;
}

gchar *writRecognDB_properties_to_insertCmds(gpointer obj){
    WritRecognDB_properties *db_prop=(WritRecognDB_properties *) obj;
    GString *sBuf = g_string_new("");
    g_string_append_printf(sBuf,"INSERT INTO %s VALUES ('%s') ;", 
	    dataTables[WRITRECOGN_TABLE_INDEX].name, db_prop->version);
    return g_string_free(sBuf, FALSE);
}


gchar *writRecognDB_properties_to_updateCmds(gpointer obj)
{
    WritRecognDB_properties *db_prop=(WritRecognDB_properties *) obj;
    GString *sBuf = g_string_new("");
    g_string_append_printf(sBuf,"UPDATE  %s SET Version = '%s' WHERE NOT NULL ;", 
	    dataTables[WRITRECOGN_TABLE_INDEX].name, db_prop->version);
    return g_string_free(sBuf, FALSE);
}

/*================ Getting results ====================*/

RadicalList *sqliteTable_append_result_radicalList
(SQLite * db, TableProperties *table, const gchar *sqlClause, RadicalList *radList){
    gchar buf[MAX_STRING_BUFFER_SIZE];
    char *zErrMsg = NULL;

    g_snprintf(buf, MAX_STRING_BUFFER_SIZE, "Reading %s\n", table->name);
    verboseMsgPrint(FILE_VERBOSE_LEVEL, buf);

    int ret = sqlite3_exec
	(db, sqlClause, radicalList_parse_result_callback, radList, &zErrMsg);
    switch(ret){
	case SQLITE_OK:
	    return radList;
	case SQLITE_ERROR:
	    /* Could be "no such column" error, skip anyway */
	    return radList;
    }
    fprintf(stderr, "In sqliteTable_append_radicalList_result(%s): ret=%d %s\n", sqlClause,ret, zErrMsg);
    sqlite3_close(db);
    return NULL;

}

RadicalArray *sqliteTable_append_result_radicalArray
(SQLite * db, TableProperties *table, const gchar *sqlClause, RadicalArray *radArray){
    gchar buf[MAX_STRING_BUFFER_SIZE];
    char *zErrMsg = NULL;

    g_snprintf(buf, MAX_STRING_BUFFER_SIZE, "Reading %s\n", table->name);
    verboseMsgPrint(FILE_VERBOSE_LEVEL, buf);

    int ret = sqlite3_exec
	(db, sqlClause, table->parse_result_callback, radArray, &zErrMsg);
    switch(ret){
	case SQLITE_OK:
	    return radArray;
	case SQLITE_ERROR:
	    /* Could be "no such column" error, skip anyway */
	    return radArray;
    }
    fprintf(stderr, "In sqliteTable_append_radicalArray_result(%s): ret=%d %s\n", sqlClause,ret, zErrMsg);
    sqlite3_close(db);
    return NULL;
}

RadicalList *sqliteCharacterDataFile_query_allTables_radicalList
(SQLite *db, const gchar *queryTemplate){
    gchar cmdBuf[MAX_STRING_BUFFER_SIZE];
    RadicalList *radList=writrecogn_radical_list_new();

    int i = 0;
    for (i = 0; i < WRITRECOGN_TABLE_INDEX; i++) {
	g_snprintf(cmdBuf, MAX_STRING_BUFFER_SIZE, queryTemplate, dataTables[i].name);
	if (sqliteTable_append_result_radicalList(db, &dataTables[i], cmdBuf, radList) == NULL) {
	    verboseMsgPrint(VERBOSE_MSG_ERROR,
		    "In sqliteCharacterDataFile_query_allTables_radicalList(): Table %s failed.\n", dataTables[i].name);
	}
    }
    return radList;
}

RadicalArray *sqliteCharacterDataFile_query_allTables_radicalArray
(SQLite *db, const gchar *queryTemplate){
    gchar cmdBuf[MAX_STRING_BUFFER_SIZE];
    RadicalArray *radArray=radicalArray_new();
    radicalArray_set_radicalType(radArray,RADICAL_TYPE_FULLCHARACTER);
    int i = 0;
    for (i = 0; i < WRITRECOGN_TABLE_INDEX; i++) {
        g_snprintf(cmdBuf, MAX_STRING_BUFFER_SIZE, queryTemplate, dataTables[i].name);
	if (sqliteTable_append_result_radicalArray(db, &dataTables[i], cmdBuf, radArray) == NULL) {
            verboseMsgPrint(VERBOSE_MSG_ERROR,
		    "In sqliteCharacterDataFile_query_allTables_radicalArray(): Table %s failed.\n", dataTables[i].name);
        }
    }
    return radArray;
}

