/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

//#ifdef HAVE_CONFIG_H
//#  include <config.h>
//#endif

#include <pthread.h>
#include <libgen.h>
#include <unistd.h>
#include <errno.h>
#include "WritRecogn.h"
#include "fileProcessing.h"
#include "utils.h"
#include "algors.h"

ProgressData *progressData_new(){
    g_debug("progressData_new\n");
    ProgressData *pdata = NEW_INSTANCE(ProgressData);
    g_assert(pdata);
    progressData_reset(pdata);
    return pdata;
}

void progressData_reset(ProgressData *pdata){
    g_debug("progressData_reset\n");
    g_assert(pdata);
    pdata->restart = TRUE;
    pdata->result = TASK_RUNNING;
    pdata->window =NULL;
    pdata->pbar =NULL;
}

void progressData_free(ProgressData *pData){
    g_free(pData);
}

TrainingProgressData *trainingProgressData_new(){
    TrainingProgressData *tProgress= NEW_INSTANCE(TrainingProgressData);
    g_assert(tProgress);
    g_debug("trainingProgressData_new\n");
    progressData_reset(&tProgress->pdata);
    tProgress->hypo=NULL;
    return tProgress;
}

void trainingProgressData_free(TrainingProgressData *tProgress){
//    progressData_free(tProgress->pdata);
    g_free(tProgress);
}


/*===========================================================
 * characterDataFile functions
 */
gchar *characterDataFile_get_preferredUserFileName()
{

    if (!isEmptyString(progAssoc.characterDataFileName)) {
        return progAssoc.characterDataFileName;
    }
    return DEFAULT_USER_CHARACTER_DATA_PATH;
}

const gchar *characterData_extensions[]={"db","xml","*",NULL};

gchar *characterDataFile_determine_filename_readable(gchar* filenameBuf){
    return filename_determine_readable(filenameBuf,
	    characterDataFile_get_preferredUserFileName(), characterData_extensions, 
	    "Open Character DataFile", progAssoc.choose_open_callback,NULL);
}

gchar *characterDataFile_determine_filename_writable(gchar* filenameBuf){
    return filename_determine_writable(filenameBuf,
	    characterDataFile_get_preferredUserFileName(), characterData_extensions, 
	    "Save Character DataFile", progAssoc.choose_save_callback,NULL);
}

/*===========================================================
 * strokeHypothesisFile functions
 */

gchar *strokeHypothesisFile_get_preferredUserFileName(){
    if (!isEmptyString(progAssoc.strokeHypothesisFileName)) {
	return progAssoc.strokeHypothesisFileName;
    }
    return DEFAULT_USER_STROKE_HYPOTHESIS_PATH;
}

const gchar *strokeHypothesisFile_extensions[]={"hypo","*",NULL};

gchar *strokeHypothesisFile_determine_filename_readable(gchar* filenameBuf){
    return filename_determine_readable(filenameBuf,
	    strokeHypothesisFile_get_preferredUserFileName(), strokeHypothesisFile_extensions, 
	    "Open Stroke Hypothesis File", progAssoc.choose_open_callback,NULL);
}

gchar *strokeHypothesisFile_determine_filename_writable(gchar* filenameBuf){
    return filename_determine_writable(filenameBuf,
	    strokeHypothesisFile_get_preferredUserFileName(), strokeHypothesisFile_extensions, 
	    "Save Stroke Hypothesis File", progAssoc.choose_save_callback,NULL);
}

TrainingProgressData *strokeHypothesisFile_start_training_TUI(StrokeRecognizer *recognizer,RadicalList *radicalList){
    if (radicalList == NULL) {
	printf("[Error] %s",  CHARACTER_DATA_NOT_LOADED_MSG);
	return NULL;
    }
    TrainingProgressData *tProgress = trainingProgressData_new();
    g_debug("strokeHypothesisFile_start_training_TUI 1\n");
    tProgress->hypo = writrecogn_stroke_recognizer_train(recognizer, radicalList);
    g_debug("strokeHypothesisFile_start_training_TUI 2\n");
    tProgress->pdata.result= TASK_COMPLETED;
    g_debug("strokeHypothesisFile_start_training_TUI -1\n");
    return tProgress;
}


//gchar *characterDataFile_get_actualUserFileName(gchar * filenameBuf, const gchar * filename, gchar * prompt,
//                                                   gboolean openMode)
//{
//    gboolean result = FALSE;

//    if (ioMsg != NULL) {
//        g_free(ioMsg);
//        ioMsg = NULL;
//    }

//    if (filename == NULL) {
//        gchar *prefFileName = characterDataFile_get_preferredUserFileName();
//        gchar *prefbuf = truepath(prefFileName, filenameBuf);
//        gchar *cdFileName;
//        if (openMode) {
//            cdFileName = progAssoc.file_open_UI(prefbuf, prompt);
//        } else {
//            /* SaveAs Mode */
//            cdFileName = progAssoc.file_saveAs_UI(prefbuf, prompt);
//        }

//        if (cdFileName == NULL) {
//            return NULL;
//        }
//        strcpy(filenameBuf, cdFileName);
//        if (filename == NULL) {
//            g_free(cdFileName);
//        }
//    } else {
//        strcpy(filenameBuf, filename);
//    }
//    return filenameBuf;
//}



//gboolean characterDataFile_open_UI(const gchar * filename)
//{
//    gchar filenameBuf[MAX_FILENAME_LENGTH];
//    GString *sBuf = g_string_new("");
//    gboolean result = FALSE;

//    gchar *cdFileName =
//        characterDataFile_get_actualUserFileName(filenameBuf, filename, "Open a characterdata File", TRUE);
//    if (cdFileName == NULL) {
//        return FALSE;
//    }
//    gchar *cdBaseFileName = g_path_get_basename(cdFileName);

//    CharacterDataFile* cdFile= writrecogn_character_datafile_new_from_file(cdFileName,FILE_OPEN_TYPE_READ_WRITE);
//    RadicalList *radList = writrecogn_radical_list_new_from_datafile(cdFile);
//    if (radList != NULL && writrecogn_radical_list_size(radList) > 0) {
//        /* Successfully open  */
//        truepath(cdFileName, progAssoc.characterDataFileName);
//        gchar *filename = g_strdup(progAssoc.characterDataFileName);
//        g_string_printf(sBuf, "%d chars in file %s loaded.", writrecogn_radical_list_size(radList),
//                        cdBaseFileName);
//        verboseMsgPrint(VERBOSE_MSG_INFO1, "%s\n", sBuf->str);
//        if (progAssoc.currRadicalList != NULL) {
//            g_object_unref(progAssoc.currRadicalList);
//        }
//        progAssoc.currRadicalList = radList;
//        result = TRUE;
//    } else {
//        g_string_printf(sBuf, "[Error] Cannot recognize the format of file %s!", cdFileName);
//        verboseMsgPrint(VERBOSE_MSG_ERROR, "%s\n", sBuf->str);
//        if (radList != NULL) {
//            g_object_unref(radList);
//        }
//        result = FALSE;
//    }

//    ioMsg = g_string_free(sBuf, FALSE);
//    g_free(cdBaseFileName);
//    return result;
//}

//gboolean characterDataFile_save_radical_UI(const gchar * filename, Radical * rad){
//    gchar filenameBuf[MAX_FILENAME_LENGTH];
//    GString *sBuf = g_string_new("");
//    gboolean result = FALSE;

//    g_debug("*** characterDataFile_save_radical_UI 1\n");

//    gchar *cdFileName =
//        characterDataFile_get_actualUserFileName(filenameBuf, filename, "Save the character data as", FALSE);
//    if (cdFileName == NULL) {
//        return FALSE;
//    }

//    g_debug("*** characterDataFile_save_radical_UI 2\n");

//    result = writrecogn_character_datafile_write_radical(cdFileName, rad);
//    g_debug("*** characterDataFile_save_radical_UI 3\n");

//    if (result) {
//        g_debug("*** characterDataFile_save_radical_UI 4\n");

//        g_strlcpy(progAssoc.characterDataFileName, cdFileName, MAX_FILENAME_LENGTH);
//        g_string_printf(sBuf, "Word %s saved to file %s.", writrecogn_radical_get_radicalCode_utf8(rad), cdFileName);
//        verboseMsgPrint(VERBOSE_MSG_INFO1, "%s\n", sBuf->str);
//    } else {
//        g_debug("*** characterDataFile_save_radical_UI 5\n");

//        g_string_printf(sBuf, "[Error] Fail to write to %s!", cdFileName);
//        verboseMsgPrint(VERBOSE_MSG_ERROR, "%s\n", sBuf->str);
//    }
//    g_debug("*** characterDataFile_save_radical_UI 6\n");

//    ioMsg = g_string_free(sBuf, FALSE);
//    return result;
//}


//gboolean characterDataFile_save_radicalList_UI(const gchar * filename, RadicalList * radList)
//{
//    gchar filenameBuf[MAX_FILENAME_LENGTH];
//    GString *sBuf = g_string_new("");
//    gboolean result = FALSE;
//    gint charCount;

//    gchar *cdFileName =
//        characterDataFile_get_actualUserFileName(filenameBuf, filename, "Save the character data as", FALSE);
//    if (cdFileName == NULL) {
//        return FALSE;
//    }
//    CharacterDataFile *cdFile=writrecogn_character_datafile_new_from_file (filename,FILE_OPEN_TYPE_WRITE_ONLY);
//    if (cdFile==NULL){
//        return FALSE;
//    }
//    gboolean success=writrecogn_character_datafile_append_radicalList(cdFile, radList);
//    if (success) {
//        g_strlcpy(progAssoc.characterDataFileName, cdFileName, MAX_FILENAME_LENGTH);
//        charCount=writrecogn_radical_list_size(radList);
//        g_string_printf(sBuf, "%d word(s) saved to file %s.", charCount, cdFileName);
//        verboseMsgPrint(VERBOSE_MSG_INFO1, "%s\n", sBuf->str);
//        result = TRUE;
//    } else {
//        g_string_printf(sBuf, "[Error] Fail to write to %s!", cdFileName);
//        verboseMsgPrint(VERBOSE_MSG_ERROR, "%s\n", sBuf->str);
//        result = FALSE;
//    }
//    ioMsg = g_string_free(sBuf, FALSE);
//    return result;
//}

// void
// saveAs_file(const char *filename) {
//     RadicalList *radList=characterDataFile_get_radicalList();
//     FullCharacter* canvasChar=canvas_get_currentFullCharacter();
//     FullCharacter* existChar=NULL;
//
//     char* _filename;
//     char buf[MAX_FILENAME_LENGTH];
//     gint ret;
//
//     if (filename==NULL) {
//         gchar *prefFileName=characterDataFile_get_preferredFileName();
//         gchar* prefbuf=truepath(prefFileName,buf);
//         gtk_file_chooser_set_filename
//             (GTK_FILE_CHOOSER(saveAsFileChooserDialogWidget),prefbuf);
//         gtk_window_set_title(GTK_WINDOW(saveAsFileChooserDialogWidget),"Save as");
//         if ((ret=gtk_dialog_run (GTK_DIALOG (saveAsFileChooserDialogWidget))) == GTK_RESPONSE_OK) {
//             _filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER
//                         (saveAsFileChooserDialogWidget));
//             gtk_widget_hide (saveAsFileChooserDialogWidget);
//         } else {
//             gtk_widget_hide (saveAsFileChooserDialogWidget);
//             return;
//         }
//
//
//         //    gtk_widget_show(openFileChooserDialogWidget);
//     } else {
//         _filename=(char *)filename;
//     }
//     if (access(_filename,W_OK)<0) {
//         g_snprintf(buf,MAX_FILENAME_LENGTH,"[Error] Cannot write to %s!",_filename);
//         verboseMsgPrint(VERBOSE_MSG_ERROR,"%s\n",buf);
//     } else {
//         if ((canvasChar->characterCode!=NULL) && (strlen(canvasChar->characterCode)>0)) {
//             existChar=radicalList_find_radical(radList,canvasChar->characterCode);
//             if (existChar!=NULL) {
//                 if (canvas_is_touched()) {
//                     /* Overwrite the FullCharacter in List using the canvas FullCharacter */
//                     radical_copy(existChar,canvasChar);
//                 } else {
//                     /* No touch, no overwrite */
//                 }
//             } else {
//                 /* Add new FullCharacter to the List */
//                 FullCharacter *newChar=g_object_new(RAW_CHARACTER_TYPE,NULL);
//                 radical_copy(newChar,canvasChar);
//                 radicalList_add_radical(radList,newChar);
//                 canvas_set_originalFullCharacter(canvasChar);
//             }
//         }
//         gint charCount=characterDataFile_write_radicalList(_filename,radList);
//         if (charCount>0) {
//             //            realpath(_filename,characterDataFileName);
//             g_strlcpy(characterDataFileName,_filename,MAX_FILENAME_LENGTH);
//             g_snprintf(buf,MAX_FILENAME_LENGTH,"%d word(s) saved to file %s.",charCount,_filename);
//             verboseMsgPrint(VERBOSE_MSG_INFO1,"%s\n",buf);
//         } else {
//
//             g_snprintf(buf,MAX_FILENAME_LENGTH,"[Error] Fail to write to %s!",_filename);
//
//             verboseMsgPrint(VERBOSE_MSG_ERROR,"%s\n",buf);
//         }
//     }
//
//     gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget),fileStatusContextId,buf);
//     if (filename==NULL) {
//         g_free (_filename);
//     }
// }


//TaskOutput strokeHypothesisFile_start_training_TUI(const gchar * filename)
//{
//    if (progAssoc.currRadicalList == NULL) {
//        printf("[Error] %s", STROKE_DATA_NOT_LOADED_MSG);
//        return FALSE;
//    }

//    ProgressData *pdata = progressData_get_instance(filename);
//    pdata->hypothesis = writrecogn_stroke_recognizer_train(progAssoc.recognizer, progAssoc.currRadicalList);
//    pdata->saved = strokeHypothesisFile_save_TUI(pdata);
//    if (pdata->saved) {
//        writrecogn_stroke_recognizer_set_hypothesis(progAssoc.recognizer, pdata->hypothesis);
//    }
//    return (pdata->saved) ? COMPLETED : FAILED;
//}

//gboolean strokeHypothesisFile_save_TUI(ProgressData * pdata)
//{
//    if (!writrecogn_stroke_recognizer_save_hypothesis(progAssoc.recognizer, pdata->hypothesis, pdata->filename)) {
//        printf("%s %s\n", FAIL_TO_SAVE_HYPOTHESIE_FILE_MSG, pdata->filename);
//        return FALSE;
//    }
//    printf("svm model saved to %s\n", pdata->filename);
//    return TRUE;
//}
