/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *  
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc. 
 * 
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com> 
 */

/**
 * InputCodeRec: Stores the input method and corresponding input code.
 */
#ifndef INPUTCODEREC_H_
#define INPUTCODEREC_H_
#include <glib.h>
#include "WritRecogn-Types.h"


gchar* 
inputMethod_to_string(InputMethod method);

/**
 * Return the corresponding input method 
 *
 * @param str the string to be parsed.
 * @return Corresponding input method, IM_INVALID if <code>str</code> is invalid input method.
 */
InputMethod 
inputMethod_parse(const char* str);

InputCodeRec*
inputCodeRec_new(InputMethod method, const gchar *inputCode);

void
inputCodeRec_free(InputCodeRec *iRec);

void 
inputCodeRec_copy(InputCodeRec *dest,InputCodeRec *src);

const char* 
inputCodeRec_to_string(InputCodeRec *self);

typedef GArray
InputCodeRecList;

InputCodeRecList*
inputCodeRecList_new();

guint
inputCodeRecList_size(InputCodeRecList *self);

gboolean inputCodeRecList_has_inputMethod(InputCodeRecList * self,InputMethod im);

/**
 * Return corresponding inputCode.
 *
 * Note: space will be used as delimiter if multiple input code records match.
 * @param self the InputCodeRecList
 * @param im the inputMethod, use IM_NOT_SPECIFIED if all input codes are required.
 * @return input code, separated by space.
 */
gchar*
inputCodeRecList_get_inputCode(InputCodeRecList *self,InputMethod im);

/**
 * inputCodeRecList_get_inputCode_array:
 * Return corresponding array of inputCode
 * @self the InputCodeRecList
 * @im the inputMethod, use IM_NOT_SPECIFIED if all input codes are required.
 *
 * A convenient call to get the array of inputCode
 * Free the array using g_strfreev()
 */
gchar**
inputCodeRecList_get_inputCode_array(InputCodeRecList *self,InputMethod im);


InputCodeRec* 
inputCodeRecList_index(InputCodeRecList *self,guint i);


void inputCodeRecList_copy(InputCodeRecList *self,InputCodeRecList *src);

void
inputCodeRecList_reset(InputCodeRecList *self);

gint 
inputCodeRecList_find_inputCodeRec_index(InputCodeRecList * self, InputCodeRec * iRec);

InputCodeRec*
inputCodeRecList_find_inputCodeRec(InputCodeRecList *self,InputCodeRec *iRec);

void inputCodeRecList_append(InputCodeRecList * self,InputCodeRec *iRec);

gboolean inputCodeRecList_remove(InputCodeRecList * self,InputCodeRec *iRec);

gchar*
inputCodeRecList_free(InputCodeRecList *self,gboolean free_segment);

#endif /*INPUTCODEREC_H_*/
