/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <limits.h>
#include <stdlib.h>
#include <libgen.h>
#include <unistd.h>
#include <errno.h>
#include <error.h>

#include "ui.h"
#include "algors.h"
#include "fileProcessing.h"
#include "file_functions.h"
#include "WritRecogn.h"
static gboolean mouseButton1Pressed = FALSE;
//static gboolean mouseButton2Pressed = FALSE;

typedef enum {
    INPUT_CODE_REC_READ,
    INPUT_CODE_REC_MODIFY,
    INPUT_CODE_REC_ADD,
    INPUT_CODE_REC_REMOVE
} InputCodeCmd;

void inputCodeRec_process_currentSelection(InputCodeCmd iCmd);

gboolean on_window_delete_event(GtkWidget * widget, GdkEvent * event, gpointer user_data) {
    quitting();
    return FALSE;
}

/*=======================================================
 * Canvas event handling functions
 */

/*
 *  This event is generated whenever the window changes size, including when it is originally created.
 */
gboolean on_canvas_configure_event(GtkWidget * widget, GdkEventConfigure * event, gpointer user_data) {
//    g_debug("on_canvas_configure_event \n");
    canvas_configure(event);
    return TRUE;
}


gboolean on_canvas_expose_event(GtkWidget * widget, GdkEventExpose * event, gpointer user_data) {
//    g_debug("on_canvas_expose_event \n");
    canvas_draw_area(event->area.x, event->area.y, event->area.width, event->area.height);
    return FALSE;
}


gboolean on_canvas_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer user_data) {
    if (event->button == 1) {
        canvas_startDrawing_rawStroke(widget, (gint) event->x, (gint) event->y);
        canvas_set_modified(TRUE);
        mouseButton1Pressed = TRUE;
    }
    return TRUE;
}


gboolean on_canvas_motion_notify_event(GtkWidget * widget, GdkEventMotion * event, gpointer user_data) {
//     g_debug("on_canvas_motion_notify_event 1 \n");
    GdkModifierType state;
    gint x = -1, y = -1;
    if (event->is_hint) {
        gdk_window_get_pointer(event->window, &x, &y, &state);
    } else {
        x = (gint) event->x;
        y = (gint) event->y;
        state = (GdkModifierType) event->state;
    }
//     g_debug("on_canvas_motion_notify_event 5 (%d,%d) \n", x, y);

    /*
     * MouseButton1Pressed is required here
     * because notify_event sometime will runs right before press_event.
     * So we need to skip such event.
     */
    if ((state & GDK_BUTTON1_MASK) && mouseButton1Pressed && canvas_is_validCoordinate(x, y)) {
//         g_debug("on_canvas_motion_notify_event 8 \n");
        canvas_continueDrawing_rawStroke(widget, x, y);
        canvas_set_modified(TRUE);
    }
//     g_debug("on_canvas_motion_notify_event 9 \n");
    return TRUE;
}

gboolean on_canvas_button_release_event(GtkWidget * widget, GdkEventButton * event, gpointer user_data) {
//     g_debug("on_canvas_button_release_event 1 \n");
    if (event->button == 1) {
        if (mouseButton1Pressed) {
//             g_debug("on_canvas_button_release_event 2 \n");
            canvas_endDrawing_rawStroke(widget, (gint) event->x, (gint) event->y);
            canvas_set_modified(TRUE);
        }
//         g_debug("on_canvas_button_release_event 5 \n");
        mouseButton1Pressed = FALSE;

    }
//     g_debug("on_canvas_button_release_event 9 \n");
    return TRUE;
}

/*=======================================================
 * Menus
 */

void on_File_New_activate(GtkMenuItem * menuitem, gpointer user_data) {
    characterDataFile_check_saved();
    if (progAssoc.characterDataFile){
	writrecogn_character_datafile_close(progAssoc.characterDataFile,FALSE);
	g_object_unref(progAssoc.characterDataFile);
    }
    gchar fileBuf[PATH_MAX];
    initString(fileBuf);
    gchar* filename=characterDataFile_determine_filename_writable(fileBuf);
    if (isEmptyString(filename))
	return;
    progAssoc.characterDataFile=writrecogn_character_datafile_new_from_file(filename,FILE_OPEN_TYPE_AUTO);
    writrecogn_radical_list_reset(progAssoc.currRadicalList);
    writrecogn_radical_list_set_backedDataFile(progAssoc.currRadicalList,G_OBJECT(progAssoc.characterDataFile));
    gtk_entry_set_text(GTK_ENTRY(wordQueryEntryWidget), "");
    canvas_reset();
}


void on_File_Open_activate(GtkMenuItem * menuitem, gpointer user_data) {
    characterDataFile_check_saved();
    characterDataFile_open_GUI();
}


void on_File_Save_activate(GtkMenuItem * menuitem, gpointer user_data) {
    characterDataFile_save_GUI(progAssoc.characterDataFile);
}

void on_File_SaveAs_activate(GtkMenuItem * menuitem, gpointer user_data) {
    characterDataFile_saveAs_GUI();
}

void on_File_Generate_Hypothesis(GtkMenuItem *menuitem,gpointer user_data){
    GtkWidget *confirmDialog = gtk_message_dialog_new(GTK_WINDOW(mainWindowWidget),
	    GTK_DIALOG_DESTROY_WITH_PARENT,
	    GTK_MESSAGE_INFO,
	    GTK_BUTTONS_YES_NO,
	    "It takes hours to generate hypothesis, proceed?");
    gint ans=gtk_dialog_run(GTK_DIALOG(confirmDialog));
    gtk_widget_destroy(confirmDialog);
    if (ans==GTK_RESPONSE_NO){
	return;
    }
    TrainingProgressData *tProgress = progAssoc.training_progress_callback
	(progAssoc.recognizer,progAssoc.currRadicalList);
    if (tProgress->pdata.result == TASK_COMPLETED) {
	gtk_entry_set_text(GTK_ENTRY(wordQueryEntryWidget), "Training Completed.");
    } else if (tProgress->pdata.result == TASK_FAILED) {
	gtk_entry_set_text(GTK_ENTRY(wordQueryEntryWidget), "Training failed.");
	verboseMsgPrint(VERBOSE_MSG_ERROR, "Training failed.\n");
	GtkWidget *cantTrainDialog = gtk_message_dialog_new(GTK_WINDOW(mainWindowWidget),
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_ERROR,
		GTK_BUTTONS_CLOSE,
		"Training failed");
	gtk_dialog_run(GTK_DIALOG(cantTrainDialog));
	gtk_widget_destroy(cantTrainDialog);
    } else  {
	/* tProgress->pdata.result == TASK_CANCELED */
	gtk_entry_set_text(GTK_ENTRY(wordQueryEntryWidget), "Training canceled.");
	verboseMsgPrint(VERBOSE_MSG_ERROR, "Training canceled.\n");
    }
}


void on_File_Open_Hypothesis(GtkMenuItem *menuitem, gpointer user_data){
    strokeHypothesisFile_open_GUI();
}

void on_File_Save_Hypothesis(GtkMenuItem *menuitem, gpointer user_data){
    strokeHypothesisFile_save_GUI(progAssoc.strokeHypothesisFileName);
}


void on_File_SaveAs_Hypothesis(GtkMenuItem *menuitem, gpointer user_data){
    strokeHypothesisFile_saveAs_GUI();
}

void on_File_Quit_activate(GtkMenuItem * menuitem, gpointer user_data) {
    quitting();
}

void on_Edit_Revert_activate(GtkMenuItem * menuitem, gpointer user_data) {
    revert_word();
}

void on_Edit_Clear_activate(GtkMenuItem * menuitem, gpointer user_data) {
    canvas_clear();
}

void on_Edit_Recognizing_select(GtkItem * item, gpointer user_data) {
    set_displayMode(RECOGNIZING_MODE);
}

void on_Edit_Editing_select(GtkItem * item, gpointer user_data) {
    set_displayMode(TRAINING_MODE);
}

gboolean on_Edit_Recognizing_button_release_event(GtkWidget * widget, GdkEventButton * event, gpointer user_data) {
    set_displayMode(RECOGNIZING_MODE);
    return FALSE;
}

gboolean on_Edit_Training_button_release_event(GtkWidget * widget, GdkEventButton * event, gpointer user_data) {
    set_displayMode(TRAINING_MODE);
    return FALSE;
}

gboolean on_Edit_UniHan_button_release_event(GtkWidget * widget, GdkEventButton * event, gpointer user_data) {
    set_displayMode(UNIHAN_MODE);
    return FALSE;
}


void on_Help_About_activate(GtkMenuItem * menuitem, gpointer user_data) {
}


void on_wordQueryEntry_activate(GtkEntry * entry, gpointer user_data) {
    const char *word = gtk_entry_get_text(GTK_ENTRY(wordQueryEntryWidget));
    display_queryWord(word);
}

/*=======================================================
 * Buttons
 */

void on_saveButton_clicked(GtkButton * button, gpointer user_data) {
    characterDataFile_save_GUI(progAssoc.characterDataFile);
}

void on_inputCodeButton_clicked(GtkButton * button, gpointer user_data) {
    GtkWidget *treeViewWidget = lookup_widget(inputCodesDialogWidget, "inputCodesDialog_treeView");
//    GtkWidget *methodComboBox = lookup_widget(inputCodesDialogWidget,
//                                "inputCodesDialog_method_comboBox");
    GtkWidget *inputCodeEntry = lookup_widget(inputCodesDialogWidget,
                                "inputCodesDialog_inputCode_entry");
    GtkListStore *listStore = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
    gint i;
    GtkTreeIter iter = { 0 };
    FullCharacter *fChar= canvas_get_current_fullCharacter();
    gint len = writrecogn_abscharacter_count_inputCodeRecs(ABSCHARACTER(fChar));

    gint ret;
    gtk_entry_set_text(GTK_ENTRY(inputCodeEntry), "");

    if (fChar && writrecogn_radical_get_radicalCode(RADICAL(fChar)) != 0) {
        for (i = 0; i < len; i++) {
	    InputCodeRec *iRec = writrecogn_abscharacter_get_inputCodeRec(ABSCHARACTER(fChar), i);
            /*
             * Add a new row to the model
             */
            gtk_list_store_append(listStore, &iter);
            gtk_list_store_set(listStore, &iter, 0, inputMethod_to_string(iRec->method), 1, iRec->inputCode, -1);
        }
        gtk_tree_view_set_model(GTK_TREE_VIEW(treeViewWidget), GTK_TREE_MODEL(listStore));



        while ((ret = gtk_dialog_run(GTK_DIALOG(inputCodesDialogWidget))) != GTK_RESPONSE_DELETE_EVENT) {
            switch (ret) {
            case GTK_RESPONSE_YES:
                // Actually, this is response from add button
		inputCodeRec_process_currentSelection(INPUT_CODE_REC_ADD);
		break;
            case GTK_RESPONSE_NO:
                // Actually, this is response from remove button
		inputCodeRec_process_currentSelection(INPUT_CODE_REC_REMOVE);
		break;
            case GTK_RESPONSE_APPLY:
		inputCodeRec_process_currentSelection(INPUT_CODE_REC_MODIFY);
                break;
            }
            if (ret == GTK_RESPONSE_CLOSE) {
                break;
            }
        }
        gtk_widget_hide(inputCodesDialogWidget);
    } else {
    }
}


void on_organizeButton_clicked(GtkButton * button, gpointer user_data) {
    FullCharacter *currentChar = canvas_get_current_fullCharacter();

    verboseMsgPrint(VERBOSE_MSG_INFO2, "Before reduce: currentChar=%s\n", 
	    writrecogn_radical_to_string(RADICAL(currentChar)));
    writrecogn_stroke_noise_reducer_process_fullCharacter(progAssoc.strokeNoiseReducer, currentChar,
	    DEFAULT_RAWWRITING_INDEX);

    verboseMsgPrint(VERBOSE_MSG_INFO2, "After reduce:currentChar=%s\n",
	    writrecogn_radical_to_string(RADICAL(currentChar)));
    canvas_redraw();
    canvas_set_modified(TRUE);
}


void on_recognizeButton_clicked(GtkButton * button, gpointer user_data) {
    FullCharacter *currentChar = canvas_get_current_fullCharacter();
    static GString *radicalBuf = NULL;
    static GString *candidateBuf = NULL;
    static GString *statusBuf = NULL;

    if (radicalBuf == NULL) {
        radicalBuf = g_string_new(NULL);
    } else {
        g_string_printf(radicalBuf, "");
    }

    if (candidateBuf == NULL) {
        candidateBuf = g_string_new(NULL);
    } else {
        g_string_printf(candidateBuf, "");
    }

    if (statusBuf == NULL) {
        statusBuf = g_string_new(NULL);
    } else {
        g_string_printf(statusBuf, "");
    }


    RawStroke *rStroke;
    Radical *radical;
    int len = writrecogn_fullcharacter_count_rawStrokes(currentChar,DEFAULT_RAWWRITING_INDEX);
    if (len <= 0) {
        g_string_printf(statusBuf, "No strokes to be recognized.");
        gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), inputCodeStatusContextId, statusBuf->str);
        return;
    }

    writrecogn_stroke_noise_reducer_process_fullCharacter(progAssoc.strokeNoiseReducer, currentChar,
	    DEFAULT_RAWWRITING_INDEX);
    if (progAssoc.displayMode != RECOGNIZING_MODE) {
        canvas_set_modified(TRUE);
    }
    canvas_redraw();
    int i;
    for (i = 0; i < len; i++) {
	rStroke = writrecogn_fullcharacter_get_rawStroke(currentChar, DEFAULT_RAWWRITING_INDEX, i);
	radical= writrecogn_stroke_recognizer_recognize(progAssoc.recognizer,rStroke);

	g_string_append_printf(radicalBuf, "%ld", 0-writrecogn_radical_get_radicalCode(radical));
    }

    RadicalList *resultList = writrecogn_character_matcher_find_matched_radicalList(progAssoc.charMatcher,
	    progAssoc.currRadicalList,currentChar);
    len = writrecogn_radical_list_size(resultList);
    for (i = 0; i < len; i++) {
	Radical *rad=writrecogn_radical_list_get_radical_by_index(resultList,i,RADICAL_TYPE_RADICAL);
        g_string_append_printf(candidateBuf, " %d%s", i + 1, writrecogn_radical_get_radicalCode_utf8(rad));
    }
    gtk_entry_set_text(GTK_ENTRY(candidateEntryWidget), candidateBuf->str);

    g_string_printf(statusBuf, "Stroke %s Recognized.", progAssoc.charMatcher->message);
    gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), inputCodeStatusContextId, statusBuf->str);
}

void on_revertButton_clicked(GtkButton * button, gpointer user_data) {
    canvas_set_modified(FALSE);
    revert_word();
}

void on_clearButton_clicked(GtkButton * button, gpointer user_data) {
    FullCharacter *currentChar = canvas_get_current_fullCharacter();
    
    writrecogn_fullcharacter_reset_rawWriting(currentChar,DEFAULT_RAWWRITING_INDEX);
    canvas_clear();
}

void on_quitButton_clicked(GtkButton * button, gpointer user_data) {
    quitting();
}

/*=======================================================
 * For inputCodeDialog
 */


void inputCodeRec_process_currentSelection(InputCodeCmd iCmd) {
    GtkWidget *inputCodeTreeView = lookup_widget(inputCodesDialogWidget, "inputCodesDialog_treeView");
    GtkWidget *methodComboBox = lookup_widget(inputCodesDialogWidget,
                                "inputCodesDialog_method_comboBox");
    GtkWidget *inputCodeEntry = lookup_widget(inputCodesDialogWidget,
                                "inputCodesDialog_inputCode_entry");

    GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(inputCodeTreeView));
    GtkTreeModel *treeModel = NULL;
    GtkTreeIter iter, dummyIter = {0};
    FullCharacter *fChar= canvas_get_current_fullCharacter();
    InputCodeRec *iRec = NULL, *iRec_ptr=NULL, iRec_tmp;

    GValue new_iMethodValue = { 0 }, new_iCodeValue = { 0 };
    InputMethod new_iMethod;
    gchar *new_iMethod_str;
    gchar *new_iCode;

    gboolean modified = FALSE;

    gboolean modelEmpty=!gtk_tree_selection_get_selected(selection, &treeModel, &iter);

    if (modelEmpty && (iCmd!=INPUT_CODE_REC_ADD)){
	/* g_tree_model has nothing. */
	return;
    }

    iRec = gtkTreeIter_to_inputCodeRec(treeModel, &iter);

    new_iMethod = gtk_combo_box_get_active(GTK_COMBO_BOX(methodComboBox));
    if (new_iMethod >= 0 && new_iMethod <= IM_OTHER) {
        new_iMethod_str = inputMethod_to_string(new_iMethod);
        g_value_init(&new_iMethodValue, G_TYPE_STRING);
        g_value_set_string(&new_iMethodValue, new_iMethod_str);
    }

    new_iCode = gtk_entry_get_text(GTK_ENTRY(inputCodeEntry));
    if (!isEmptyString(new_iCode)) {
        g_value_init(&new_iCodeValue, G_TYPE_STRING);
        g_value_set_string(&new_iCodeValue, new_iCode);
    }
    switch (iCmd) {
    case INPUT_CODE_REC_READ:
        gtk_combo_box_set_active(GTK_COMBO_BOX(methodComboBox), iRec->method);
        gtk_entry_set_text(GTK_ENTRY(inputCodeEntry), iRec->inputCode);
        break;
    case INPUT_CODE_REC_ADD:
	if (isEmptyString(new_iCode)) {
	    break;
	}
	iRec_tmp.method=new_iMethod;
	iRec_tmp.inputCode=new_iCode;
	iRec_ptr=writrecogn_abscharacter_find_inputCodeRec(ABSCHARACTER(fChar),&iRec_tmp);
	if (iRec_ptr){
	    // Duplicate found.
	    break;
	}
	modified=TRUE;
	gtk_list_store_append(GTK_LIST_STORE(treeModel), &iter);
	gtk_list_store_set(GTK_LIST_STORE(treeModel), &iter, 0, inputMethod_to_string(new_iMethod), 1,new_iCode , -1);
	writrecogn_abscharacter_append_inputCodeRec(ABSCHARACTER(fChar),&iRec_tmp);
	break;
    case INPUT_CODE_REC_REMOVE:
	if (!writrecogn_abscharacter_remove_inputCodeRec(ABSCHARACTER(fChar),iRec)){
	    break;
	}
	modified=TRUE;
	gtk_list_store_remove(GTK_LIST_STORE(treeModel), &iter);
	gtk_entry_set_text(GTK_ENTRY(inputCodeEntry), "");	
	break;
    case INPUT_CODE_REC_MODIFY:
	if (iRec->method != new_iMethod) {
            modified = TRUE;
            gtk_list_store_set_value(GTK_LIST_STORE(treeModel), &iter, 0, &new_iMethodValue);
        }
        if (!isEmptyString(new_iCode)) {
            if (iRec && strcmp(iRec->inputCode, new_iCode)) {
                modified = TRUE;
                gtk_list_store_set_value(GTK_LIST_STORE(treeModel), &iter, 1, &new_iCodeValue);
            }
        }
        if (modified) {
	    writrecogn_abscharacter_reset_inputCodeRecList(ABSCHARACTER(fChar));
	    iter = dummyIter;
	    if (gtk_tree_model_get_iter_first(treeModel, &iter)) {
		do {
		    iRec_ptr = gtkTreeIter_to_inputCodeRec(treeModel,
			    &iter);
		    if (iRec_ptr) {
			writrecogn_abscharacter_append_inputCodeRec(ABSCHARACTER(fChar),iRec_ptr);
			g_free(iRec_ptr);
		    }
		} while (gtk_tree_model_iter_next(treeModel, &iter));
	    }
        }
        break;
    }
    if (modified){
	writrecogn_radical_list_update_radical_datafile(progAssoc.currRadicalList,
		RADICAL(fChar),RADICAL_TYPE_ABSCHARACTER);
	verboseMsgPrint(FILE_VERBOSE_LEVEL,writrecogn_radical_to_string(RADICAL(fChar)));
	progAssoc.flags.characterDataModified=progAssoc.characterDataFile->modified;
	windowTitle_update();
    }
    if (iRec) {
        inputCodeRec_free(iRec);
    }

}

void on_inputCodesDialog_treeView_cursor_changed(GtkTreeView * treeview, gpointer user_data) {
    inputCodeRec_process_currentSelection(INPUT_CODE_REC_READ);
}

void on_inputCodesDialog_inputCode_entry_activate(GtkEntry * entry, gpointer user_data) {
//    inputCodeRec_process_currentSelection(INPUT_CODE_REC_MODIFY);
}



