/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */


#include <stdlib.h>
#include <glib.h>
#include <math.h>
#include "verboseMsg.h"
#include "ui.h"
#include "WritRecogn.h"
#define  STROKE_COORDINATE_VERBOSE_LEVEL VERBOSE_MSG_INFO2

GdkColor canvasBgColor, gridLineFgColor, drawingFgColor, practicingCharacterFgColor;
GdkGC *gridLinesGdkGC;
GdkGC *canvasGdkGC;
GdkGC *drawingGdkGC;
GdkGC *practicingCharacterGdkGC;

GdkColormap *sysColorMap=NULL;
//GdkGCValues *gridLinesGdkGCValues;
GdkPixmap *pixmap;

gboolean canvasIsModified = FALSE;
int canvasHeight = -1;
int canvasWidth = -1;
int horizontalGridLines = 2;
int verticalGridLines = 2;
int penTipWidth = DEFAULT_PEN_TIP_WIDTH;
int strokeNodeWidth = DEFAULT_STROKE_NODE_WIDTH;
gint penX=-1;
gint penY=-1;
gint prevPenX = -1;
gint prevPenY = -1;
gboolean showCoordinateStatus = TRUE;

FullCharacter *curr_fChar, *orig_fChar;
RawStroke *curr_rawStroke, *orig_rawStroke;
float xScale = -1.0f;
float yScale = -1.0f;

PangoLayout *practicingCharacterPangoLayout = NULL;
PangoFontDescription *practicingCharacterFontDesc = NULL;

guint penStatusContextId;
guint inputCodeStatusContextId;
guint fileStatusContextId;


void canvas_init() {
    canvasBgColor.red = canvasBgColor.blue = canvasBgColor.green = 57343;
    gridLineFgColor.red = gridLineFgColor.blue = gridLineFgColor.green = 49151;
    drawingFgColor.red = drawingFgColor.blue = drawingFgColor.green = 0;
    practicingCharacterFgColor.red = practicingCharacterFgColor.blue = 32767;
    practicingCharacterFgColor.green = 32767;

    canvas_set_showCoordinateStatus(TRUE);

    sysColorMap=gdk_colormap_get_system();

    penStatusContextId = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusBarWidget), "Pen Status");
    inputCodeStatusContextId = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusBarWidget), "Input code");
    fileStatusContextId = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusBarWidget), "File");

    curr_fChar = writrecogn_fullcharacter_new();
    writrecogn_fullcharacter_new_rawWriting(curr_fChar);
    orig_fChar = writrecogn_fullcharacter_new();
    writrecogn_fullcharacter_new_rawWriting(orig_fChar);
    practicingCharacterFontDesc = pango_font_description_new();
}

/* on_canvas_configure_event will call this function */
void canvas_configure(GdkEventConfigure * event) {
    if ((canvasWidth != event->width) || (canvasHeight != event->height) || (pixmap == NULL)) {
	if (pixmap) {
	    g_object_unref(pixmap);
        }
	canvasWidth = event->width;
        canvasHeight = event->height;

        GdkWindow *parentWindow=gtk_widget_get_parent_window(canvasWidget);
        pixmap = gdk_pixmap_new(GDK_DRAWABLE(parentWindow), event->width, event->height, -1);
    }


    canvasGdkGC = gdk_gc_new(pixmap);
    gridLinesGdkGC = gdk_gc_new(pixmap);
    drawingGdkGC = gdk_gc_new(pixmap);
    practicingCharacterGdkGC = gdk_gc_new(pixmap);

    gdk_gc_set_colormap(canvasGdkGC,sysColorMap);
    gdk_gc_set_rgb_fg_color(canvasGdkGC, &canvasBgColor);
    gdk_gc_set_rgb_bg_color(canvasGdkGC, &canvasBgColor);

    gdk_gc_set_colormap(gridLinesGdkGC,sysColorMap);
    gdk_gc_set_rgb_fg_color(gridLinesGdkGC, &gridLineFgColor);
    gdk_gc_set_rgb_bg_color(gridLinesGdkGC, &canvasBgColor);
    gdk_gc_set_line_attributes(gridLinesGdkGC, 2, GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_MITER);

    gdk_gc_set_colormap(drawingGdkGC,sysColorMap);
    gdk_gc_set_rgb_fg_color(drawingGdkGC, &drawingFgColor);
    gdk_gc_set_rgb_bg_color(drawingGdkGC, &canvasBgColor);
    gdk_gc_set_line_attributes(drawingGdkGC, penTipWidth, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);

    gdk_gc_set_colormap(practicingCharacterGdkGC,sysColorMap);
    gdk_gc_set_rgb_fg_color(practicingCharacterGdkGC, &practicingCharacterFgColor);
    gdk_gc_set_rgb_bg_color(practicingCharacterGdkGC, &canvasBgColor);


    /* Pango Setting */
    pango_font_description_set_absolute_size(practicingCharacterFontDesc,
            PANGO_SCALE * (canvasHeight - 2 * PRACTICING_CHARACTER_SPACING));


    if (practicingCharacterPangoLayout == NULL) {
        practicingCharacterPangoLayout = gtk_widget_create_pango_layout(canvasWidget, "田");
    }

    pango_layout_set_width(practicingCharacterPangoLayout, PANGO_SCALE * (canvasWidth - 2 * PRACTICING_CHARACTER_SPACING));
    pango_layout_set_font_description(practicingCharacterPangoLayout, practicingCharacterFontDesc);
    pango_layout_set_alignment(practicingCharacterPangoLayout, PANGO_ALIGN_CENTER);
}


void canvas_draw_area(gint x, gint y, gint width, gint height) {
    if (!canvas_is_x_valid(x)) {
        return;
    }
    if (!canvas_is_y_valid(y)) {
        return;
    }
    gdk_draw_drawable(canvasWidget->window, canvasWidget->style->fg_gc[GTK_WIDGET_STATE(canvasWidget)], pixmap, x, y, x, y, width, height);
}

void canvas_reset() {
//    g_debug("canvas_reset \n");
    canvas_reset_original_fullCharacter();
    canvas_reset_current_fullCharacter();
    canvas_redraw();
    canvas_set_modified(FALSE);
}

void canvas_redraw() {
    if (!progAssoc.flags.initializedFlag) {
	// Because widgets are not ready.
	return;
    }
//    g_debug("canvas_redraw \n");

    canvas_draw_backgroundPixmap();
    canvas_draw_foregroundPixmap();
    gint cW, cH;
    gdk_drawable_get_size(pixmap, &cW, &cH);
    xScale = (float) canvas_get_width() / FULL_CHARACTER_WIDTH;
    yScale = (float) canvas_get_height() / FULL_CHARACTER_HEIGHT;
    canvas_draw_area(0, 0, cW, cH);
}


void canvas_clear() {
    writrecogn_radical_reset(RADICAL(curr_fChar));
    canvas_redraw();
}

void canvas_draw_foregroundPixmap() {
    const gchar *wordQuery = gtk_entry_get_text(GTK_ENTRY(wordQueryEntryWidget));
    if (!isEmptyString(wordQuery)) {
        pango_layout_set_text(practicingCharacterPangoLayout, wordQuery, MAX_CHAR_CODE_LENGTH);
        gdk_draw_layout(pixmap, practicingCharacterGdkGC, PRACTICING_CHARACTER_SPACING, 0, practicingCharacterPangoLayout);
    }
    canvas_draw_fullCharacter(curr_fChar);
}

void canvas_draw_backgroundPixmap() {
    gdk_draw_rectangle(pixmap, canvasGdkGC, TRUE, 0, 0, canvasWidth, canvasHeight);
    // Draw grid lines


    int i;
    for (i = 1; i < horizontalGridLines + 1; i++) {
        gdk_draw_line(pixmap, gridLinesGdkGC, 
		0, canvasHeight * i / (horizontalGridLines + 1), 
		canvasWidth - 1, canvasHeight * i / (horizontalGridLines + 1));
    }
    for (i = 1; i < verticalGridLines + 1; i++) {
        gdk_draw_line(pixmap, gridLinesGdkGC,
		canvasWidth * i / (verticalGridLines + 1), 
		0, canvasWidth * i / (verticalGridLines + 1), canvasHeight - 1);
    }
}

gboolean canvas_is_x_valid(gint x) {
    if (x > canvasWidget->allocation.width - 1 || x < 0) {
        return FALSE;
    }
    return TRUE;
}

gboolean canvas_is_y_valid(gint y) {
    if (y > canvasWidget->allocation.height - 1 || y < 0) {
        return FALSE;
    }
    return TRUE;
}

gboolean canvas_is_validCoordinate(gint x, gint y) {
    if (!canvas_is_x_valid(x) || !canvas_is_y_valid(y)) {
        return FALSE;
    }
    return TRUE;
}

gint canvas_get_width() {
    gdk_drawable_get_size(pixmap, &canvasWidth, &canvasHeight);
    return canvasWidth;
}

gint canvas_get_height() {
    gdk_drawable_get_size(pixmap, &canvasWidth, &canvasHeight);
    return canvasHeight;
}

gboolean canvas_is_showCoordinateStatus() {
    return showCoordinateStatus;
}

void canvas_set_showCoordinateStatus(gboolean _showCoordinateStatus) {
    showCoordinateStatus = _showCoordinateStatus;
}

void canvas_draw_point(GtkWidget * widget, gint x, gint y) {
    GdkRectangle update_rect;
    update_rect.x = x - strokeNodeWidth / 2;
    update_rect.y = y - strokeNodeWidth / 2;
    update_rect.width = strokeNodeWidth;
    update_rect.height = strokeNodeWidth;

    gdk_draw_rectangle(pixmap, drawingGdkGC, TRUE, 
	    update_rect.x, update_rect.y, update_rect.width, update_rect.height);
    gtk_widget_queue_draw_area(widget, 
	    update_rect.x, update_rect.y, update_rect.width, update_rect.height);
    gchar buf[40];
    g_sprintf(buf, "x: %d, y: %d", x, y);
    verboseMsgPrint(STROKE_COORDINATE_VERBOSE_LEVEL, "[INFO] drawPoint: %s\n", buf);
    if (canvas_is_showCoordinateStatus()) {
        gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), penStatusContextId, buf);
    }
}

void canvas_draw_line(GtkWidget * widget, gint xStart, gint yStart, gint xEnd, gint yEnd) {
    GdkRectangle update_rect;
    update_rect.x = MIN(xStart, xEnd);
    update_rect.y = MIN(yStart, yEnd);
    update_rect.width = abs(xStart - xEnd) + 1;
    update_rect.height = abs(yStart - yEnd) + 1;

    gdk_draw_line(pixmap, drawingGdkGC, xStart, yStart, xEnd, yEnd);
    gtk_widget_queue_draw_area(widget, 
	    update_rect.x, update_rect.y, update_rect.width, update_rect.height);
    gchar buf[60];
    g_sprintf(buf, "xStart: %d, yStart: %d, xEnd: %d, yEnd: %d", xStart, yStart, xEnd, yEnd);
    verboseMsgPrint(STROKE_COORDINATE_VERBOSE_LEVEL, "[INFO] drawLine: %s\n", buf);
    if (canvas_is_showCoordinateStatus()) {
        gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), penStatusContextId, buf);
    }
}


guint canvas_add_rawStrokeNode(RawStroke * rStroke, gint x, gint y) {
    gint xAct = TO_STORE_COORDINATE(x, xScale);
    gint yAct = TO_STORE_COORDINATE(y, yScale);
    guint newLen=writrecogn_rawstroke_add_rawStrokeNode(rStroke, xAct, yAct);
    return newLen;
}

void canvas_startDrawing_rawStroke(GtkWidget * widget, gint x, gint y) {
    prevPenX=prevPenY=-1;
    penX = x;
    penY = y;
    canvas_draw_point(widget, x, y);
    /* Add RawStroke and RawStrokeNode */
    curr_rawStroke = writrecogn_rawstroke_new();
    canvas_add_rawStrokeNode(curr_rawStroke, x, y);
}

void canvas_continueDrawing_rawStroke(GtkWidget * widget, gint x, gint y) {
    if (!canvas_is_validCoordinate(x, y)) {
        return;
    }
    prevPenX=penX;
    prevPenY=penY;
    penX = x;
    penY = y;

    canvas_draw_line(widget, prevPenX, prevPenY, penX, penY);
    canvas_draw_point(widget, penX, penY);

    /* Add  RawStrokeNode */
    canvas_add_rawStrokeNode(curr_rawStroke, penX, penY);
}

void canvas_endDrawing_rawStroke(GtkWidget * widget, gint x, gint y) {
    canvas_continueDrawing_rawStroke(widget,x,y);
    writrecogn_fullcharacter_add_rawStroke(curr_fChar,DEFAULT_RAWWRITING_INDEX,curr_rawStroke);
    prevPenX=prevPenY=penX=penY=-1;
}

FullCharacter *canvas_get_current_fullCharacter() {
    return curr_fChar;
}

void canvas_reset_current_fullCharacter() {
    writrecogn_radical_reset(RADICAL(curr_fChar));
}

void canvas_set_current_fullCharacter(FullCharacter * fChar) {
    writrecogn_radical_copy(RADICAL(curr_fChar), RADICAL(fChar));
}

FullCharacter *canvas_get_original_fullCharacter() {
    return orig_fChar;
}

void canvas_reset_original_fullCharacter() {
    writrecogn_radical_reset(RADICAL(orig_fChar));
}

void canvas_set_original_fullCharacter(FullCharacter * fChar) {
    writrecogn_radical_copy(RADICAL(orig_fChar), RADICAL(fChar));
}

void canvas_draw_rawStroke(RawStroke * rawStroke) {
    canvas_set_showCoordinateStatus(FALSE);
    showCoordinateStatus = FALSE;
    int i;
    RawStrokeNode *node, *nextNode;
    int len = writrecogn_rawstroke_count_rawStrokeNodes(rawStroke);
    gint xStart, yStart, xEnd, yEnd;
    for (i = 0; i < len - 1; i++) {
	node = writrecogn_rawstroke_get_rawStrokeNode(rawStroke,i);
	nextNode = writrecogn_rawstroke_get_rawStrokeNode(rawStroke,i+1);
        xStart = TO_DISPLAY_COORDINATE(node->x, xScale);
        yStart = TO_DISPLAY_COORDINATE(node->y, yScale);
        xEnd = TO_DISPLAY_COORDINATE(nextNode->x, xScale);
        yEnd = TO_DISPLAY_COORDINATE(nextNode->y, yScale);
        canvas_draw_line(canvasWidget, xStart, yStart, xEnd, yEnd);
        canvas_draw_point(canvasWidget, xStart, yStart);
        canvas_draw_point(canvasWidget, xEnd, yEnd);
    }
}


void canvas_draw_fullCharacter(FullCharacter * fChar) {
    int i;
    int rawWritingIndex=DEFAULT_RAWWRITING_INDEX;
    int len = writrecogn_fullcharacter_count_rawStrokes (fChar,rawWritingIndex);
    for (i = 0; i < len; i++) {
	RawStroke *rStroke = writrecogn_fullcharacter_get_rawStroke(fChar,rawWritingIndex, i);
        canvas_draw_rawStroke(rStroke);
    }
}

gboolean canvas_is_modified() {
    return canvasIsModified;
}

void canvas_set_modified(gboolean modified) {
    progAssoc.flags.canvasModified=canvasIsModified = modified;
    if (progAssoc.displayMode!=RECOGNIZING_MODE) {
        windowTitle_update();
    }
}

