/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */


/**
 * Canvas & display functions
 */
#ifndef CANVAS_H_
#define CANVAS_H_
#include <gtk/gtk.h>
#include <math.h>
#include "WritRecogn.h"

/**
 * Default pen tip width
 */
#define DEFAULT_PEN_TIP_WIDTH 2
#define DEFAULT_STROKE_NODE_WIDTH 6
#define PRACTICING_CHARACTER_SPACING 10
#define CANVAS_MIN_HEIGHT 20
#define CANVAS_MIN_WIDTH 20
#define FULL_CHARACTER_WIDTH 1000
#define FULL_CHARACTER_HEIGHT 1000

#define TO_DISPLAY_COORDINATE(x,scale)  FLOAT_TO_INT(x*scale)
#define TO_STORE_COORDINATE(x,scale)  FLOAT_TO_INT(x/scale)

int canvasHeight;
int canvasWidth;
int horizontalGridLines;
int verticalGridLines;

GtkDrawingArea *canvas;
GtkWidget *canvasWidget;

int penTipWidth;

void canvas_init();

void canvas_configure(GdkEventConfigure * event);

/**
 * Clear and redraw the canvas
 */
void canvas_draw_area(gint x, gint y, gint width, gint height);

void canvas_reset();

void canvas_redraw();

void canvas_clear();

void canvas_draw_foregroundPixmap();

void canvas_draw_backgroundPixmap();

gboolean canvas_is_ValidCoordinate(gint x, gint y);

gint canvas_get_width();

gint canvas_get_height();

gboolean canvas_is_showCoordinateStatus();

void canvas_set_showCoordinateStatus(gboolean _showCoordinateStatus);

gboolean canvas_is_x_valid(gint x);

gboolean canvas_is_y_valid(gint y);

gboolean canvas_is_validCoordinate(gint x, gint y);

/**
 * Start stroke when user press mouse button-1 on canvas (left button)
 * @param widget	activated canvas widget
 * @param x Current mouse x coordinate
 * @param y	Current mouse y coordinate
 */
void canvas_draw_point(GtkWidget * widget, gint x, gint y);

void canvas_draw_line(GtkWidget * widget, gint xStart, gint yStart, gint xEnd, gint yEnd);

void canvas_startDrawing_rawStroke(GtkWidget * widget, gint x, gint y);

void canvas_continueDrawing_rawStroke(GtkWidget * widget, gint x, gint y);

void canvas_endDrawing_rawStroke(GtkWidget * widget, gint x, gint y);

FullCharacter *canvas_get_current_fullCharacter();

void canvas_reset_current_fullCharacter();


/**
 * canvas_set_current_fullCharacter:
 * Set the current FullCharacter for the canvas to display and process.
 * @fChar: the FullCharacter to be set.
 *
 * Note that this function will not redraw the canvas, call canvas_redraw() after this function.
 */
void canvas_set_current_fullCharacter(FullCharacter *fChar);

FullCharacter *canvas_get_original_fullCharacter();

void canvas_reset_original_fullCharacter();

/**
 * canvas_set_original_fullCharacter:
 * Set the FullCharacter for the canvas for revert.
 * @fChar: the FullCharacter to be set.
 */
void canvas_set_original_fullCharacter(FullCharacter *fChar);

void canvas_draw_rawStroke(RawStroke * rawStroke);


void canvas_draw_fullCharacter(FullCharacter * fChar);

gboolean canvas_is_modified();



/**
 * Set when canvas is  modified by drawing or clearing.
 *
 * Note that in RECOGNIZER_MODE, the window title will not set to be modified.
 * @modified: Whether the canvas is modified
 */
void canvas_set_modified(gboolean modified);

#endif				/*CANVAS_H_ */
