/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

#include <pthread.h>
#include <libgen.h>
#include <unistd.h>
#include <errno.h>
#include <gtk/gtk.h>
#include "support.h"
#include "WritRecogn-Types.h"
#include "file_functions.h"
#include "fileProcessing.h"
#include "algors.h"
#include "ui.h"

pthread_t trainer_tid;
//ProgressData *pdata;

void strokeHypothesisFile_finish_training(gpointer data);

gchar *filename_get_user_chosen_callback_open_GUI
(const gchar *defaultFileName, const gchar** extensions,const gchar * prompt,gpointer option){
    //printf("***prefFileName=%s buf=%s prefbuf=%s\n", prefFileName,buf,prefbuf);
    gchar *newFileName = NULL;
    char buf[PATH_MAX];

    int ret;
    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(openFileChooserDialogWidget), defaultFileName);
    gtk_window_set_title(GTK_WINDOW(openFileChooserDialogWidget), prompt);
    while ((ret = gtk_dialog_run(GTK_DIALOG(openFileChooserDialogWidget))) != GTK_RESPONSE_CANCEL) {
	gchar *_filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(openFileChooserDialogWidget));
	if (isReadable(_filename)) {
	    newFileName = _filename;
	    break;
	} else {
	    g_snprintf(buf, PATH_MAX, "[Error] Cannot read %s - check path and permission.", _filename);
	    verboseMsgPrint(VERBOSE_MSG_ERROR, "%s\n", buf);
	    GtkWidget *cantReadDialog = gtk_message_dialog_new(GTK_WINDOW(openFileChooserDialogWidget),
		    GTK_DIALOG_DESTROY_WITH_PARENT,
		    GTK_MESSAGE_ERROR,
		    GTK_BUTTONS_CLOSE,
		    "%s", buf);
	    gtk_dialog_run(GTK_DIALOG(cantReadDialog));
	    gtk_widget_destroy(cantReadDialog);
	}
    }
    gtk_widget_hide(openFileChooserDialogWidget);
    return newFileName;
}

gchar *filename_get_user_chosen_callback_save_GUI
(const gchar *defaultFileName, const gchar** extensions, const gchar * prompt,gpointer option){
    gchar *newFileName = NULL;
    gchar buf[PATH_MAX];

    int ret;
    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(saveAsFileChooserDialogWidget), defaultFileName);
    gtk_window_set_title(GTK_WINDOW(saveAsFileChooserDialogWidget), prompt);
    while ((ret = gtk_dialog_run(GTK_DIALOG(saveAsFileChooserDialogWidget))) != GTK_RESPONSE_CANCEL) {
	gchar *_filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(saveAsFileChooserDialogWidget));
	if (!isWritable(_filename)) {
	    g_snprintf(buf, PATH_MAX, "[Error] Cannot write %s - check path and permission.", _filename);
	    verboseMsgPrint(VERBOSE_MSG_ERROR, "%s\n", buf);
	    GtkWidget *cantWriteDialog = gtk_message_dialog_new(GTK_WINDOW(saveAsFileChooserDialogWidget),
		    GTK_DIALOG_DESTROY_WITH_PARENT,
		    GTK_MESSAGE_ERROR,
		    GTK_BUTTONS_CLOSE,
		    "%s", buf);
	    gtk_dialog_run(GTK_DIALOG(cantWriteDialog));
	    gtk_widget_destroy(cantWriteDialog);
	    g_free(_filename);
	    continue;
	}
	newFileName = _filename;
	break;
    }
    gtk_widget_hide(saveAsFileChooserDialogWidget);
    return newFileName;
}

gboolean characterDataFile_open_GUI() {
    gchar fileBuf[PATH_MAX];
    initString(fileBuf);
    fileBuf[0]='\0';
    GString *strBuf=g_string_new(NULL);
    CharacterDataFile *cDataFile;
    RadicalList *radList;
    gboolean pass=FALSE;
    characterDataFile_check_saved();

    gchar* filename=characterDataFile_determine_filename_readable(fileBuf);
    if (isEmptyString(filename)){
	g_string_printf(strBuf,"No filename. ");
    }else if((cDataFile=writrecogn_character_datafile_new_from_file(filename,FILE_OPEN_TYPE_READ_WRITE))==NULL){
	g_string_printf(strBuf,"Cannot new CharacterDataFile %s ",filename);
    }else if (!writrecogn_character_datafile_open(cDataFile)){
	g_string_printf(strBuf,"Cannot open CharacterDataFile %s ",filename);
    }else if ((radList=writrecogn_character_datafile_read_radicalList(cDataFile))==NULL){
	g_string_printf(strBuf,"Cannot read CharacterDataFile %s ",filename);
    }else{
	gtk_entry_set_text(GTK_ENTRY(wordQueryEntryWidget), "");
	canvas_reset();
	progAssoc.characterDataFile=cDataFile;
	progAssoc.currRadicalList=radList;
	pass = TRUE;
	progAssoc.flags.characterDataModified=FALSE;
	canvas_set_modified(FALSE);
	windowTitle_update();
    }
    gchar *ioMsg=cDataFile->message;
    if (!isEmptyString(ioMsg)){
        g_string_append_printf(strBuf," ioMsg: %s",ioMsg);
    }
    gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, strBuf->str);
    g_string_free(strBuf,FALSE);
    return pass;
}

gboolean characterDataFile_save_GUI(CharacterDataFile *cDataFile) {
    GString *strBuf=g_string_new(NULL);
    gboolean pass=FALSE;
    FullCharacter *canvasChar = canvas_get_current_fullCharacter();
    RadicalCode radicalCode=RADICAL(canvasChar)->radicalCode;
    if (!cDataFile->modified) {
	g_string_printf(strBuf,"No changes, no save.");
	canvas_set_modified(false);
	progAssoc.flags.characterDataModified=FALSE;
    }else  if ((radicalCode!=0) && 
	    !writrecogn_character_datafile_update_radical(cDataFile,RADICAL(canvasChar))){
	g_string_printf(strBuf,"Cannot update radical %s  in CharacterDataFile %s ",
		writrecogn_radical_get_radicalCode_utf8(RADICAL(canvasChar)),
		writrecogn_character_datafile_get_filename(cDataFile));
    }else if (!writrecogn_character_datafile_flush(cDataFile)){
	g_string_printf(strBuf,"Cannot flush datafile.");
    }else{
	g_string_printf(strBuf,"Char data saved.");
	pass=TRUE;
    }
    gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, strBuf->str);
    g_string_free(strBuf,TRUE);
    if (pass){
	progAssoc.flags.characterDataModified=FALSE;
	canvas_set_modified(FALSE);
	gchar *filename=writrecogn_character_datafile_get_filename(cDataFile);
	if (strcmp(progAssoc.characterDataFileName,filename)!=0){
	    strcpy(progAssoc.characterDataFileName,filename);
	}
	windowTitle_update();
    }
    return pass;
}

gboolean characterDataFile_saveAs_GUI() {
    gboolean pass = FALSE;
    gchar fileBuf[PATH_MAX];
    initString(fileBuf);
    GString *strBuf=g_string_new(NULL);
    CharacterDataFile *cDataFile;
    gboolean poped=FALSE;

    gchar* filename=characterDataFile_determine_filename_writable(fileBuf);
    if (isEmptyString(filename)){
	g_string_printf(strBuf,"No filename. ");
    }else if((cDataFile=writrecogn_character_datafile_new_from_file(filename,FILE_OPEN_TYPE_WRITE_ONLY))==NULL){
	g_string_printf(strBuf,"Cannot new CharacterDataFile %s ",filename);
    }else if (!writrecogn_character_datafile_open(cDataFile)){
	g_string_printf(strBuf,"Cannot open CharacterDataFile %s ",filename);
    }else if (!writrecogn_character_datafile_append_radicalList(cDataFile,progAssoc.currRadicalList)){
	g_string_printf(strBuf,"Cannot append current radical list to CharacterDataFile %s ",filename);
    }else if (!characterDataFile_save_GUI(cDataFile) ){
	poped=TRUE;
    }else{
	writrecogn_character_datafile_close(progAssoc.characterDataFile,FALSE);
	writrecogn_character_datafile_close(cDataFile,FALSE);
	g_object_unref(progAssoc.characterDataFile);
//        g_object_unref(progAssoc.currRadicalList);
	writrecogn_character_datafile_set_openType(cDataFile,FILE_OPEN_TYPE_READ_WRITE);
	writrecogn_character_datafile_open(cDataFile);
	progAssoc.characterDataFile=cDataFile;
	progAssoc.currRadicalList=writrecogn_character_datafile_read_radicalList(cDataFile);
	pass = TRUE;
    }

    gchar *ioMsg=cDataFile->message;
    if (!isEmptyString(ioMsg)){
        g_string_append_printf(strBuf," ioMsg: %s",ioMsg);
    }
    if (!pass){
	if (!poped){
	    gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, strBuf->str);
	}
    }else{
	/* Successful message already shown in save_GUI() */
    }
    g_string_free(strBuf,TRUE);
    return pass;
}

/*===================================================
 * Stroke hypothesis file functions
 */

gboolean strokeHypothesisFile_open_GUI() {
    gchar fileBuf[PATH_MAX];
    initString(fileBuf);
    fileBuf[0]='\0';
    GString *strBuf=g_string_new(NULL);
    gboolean pass=FALSE;

    strokeHypothesisFile_check_saved();
    gchar* filename=strokeHypothesisFile_determine_filename_readable(fileBuf);
    if (isEmptyString(filename)){
	g_string_printf(strBuf,"No hypothesis filename. ");
    }else if (!writrecogn_stroke_recognizer_load_hypothesis(progAssoc.recognizer,filename)){
	g_string_printf(strBuf,"Fail reading hypothesis %s ",filename);
    }else{
	strcpy(progAssoc.strokeHypothesisFileName,filename);
	pass = TRUE;
	progAssoc.flags.hypothesisModified=FALSE;
	windowTitle_update();
    }
    gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, strBuf->str);
    g_string_free(strBuf,FALSE);
    return pass;
}


gboolean strokeHypothesisFile_save_GUI(const gchar* filename) {
    if (!writrecogn_stroke_recognizer_save_hypothesis(progAssoc.recognizer, filename)) {
	GtkWidget *cantWriteDialog = gtk_message_dialog_new(GTK_WINDOW(progressBarDialogWidget),
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_ERROR,
		GTK_BUTTONS_CLOSE,
		"%s %s", FAIL_TO_SAVE_HYPOTHESIE_FILE_MSG, filename);
	gtk_dialog_run(GTK_DIALOG(cantWriteDialog));
	gtk_widget_destroy(cantWriteDialog);
	return FALSE;
    }
    gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, "Hypothesis saved.");
    progAssoc.flags.hypothesisModified=FALSE;
    if (strcmp(progAssoc.strokeHypothesisFileName,filename)!=0){
	strcpy(progAssoc.strokeHypothesisFileName,filename);
    }
    windowTitle_update();
    return TRUE;
}

gboolean strokeHypothesisFile_saveAs_GUI() {
    gboolean pass = FALSE;
    gchar fileBuf[PATH_MAX];
    fileBuf[0]='\0';
    GString *strBuf=g_string_new(NULL);

    gchar* filename=strokeHypothesisFile_determine_filename_writable(fileBuf);
    if (isEmptyString(filename)){
	g_string_printf(strBuf,"No hypothesis filename. ");
    }else if(!strokeHypothesisFile_save_GUI(filename)){
    }else {
	pass=TRUE;
    
    }
    if (!pass){
	gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, strBuf->str);
    }else{
	/* Successful message already shown in save_GUI() */
    }
    g_string_free(strBuf,TRUE);
    return pass;
}

/*===================================================
 * Stroke hypothesis training functions
 */

/* Clean up allocated memory and remove the timer */
void destroy_progressBar(ProgressData * pdata) {
    if (pdata->timer > 0) {
        g_source_remove(pdata->timer);
        pdata->timer = 0;
        /*    pdata->window = NULL;*/
        g_free(pdata);
    }
}

void *trainer_thread(gpointer data) {
    TrainingProgressData *tProgress = (TrainingProgressData *) data;
    tProgress->hypo = writrecogn_stroke_recognizer_train(progAssoc.recognizer, progAssoc.currRadicalList);
    tProgress->pdata.result=TASK_COMPLETED;
    pthread_exit(0);
}

// gpointer
// trainer_thread(gpointer data) {
//     ProgressData *pdata = (ProgressData *)data;
//     pdata->hypothesis= strokeRecognizer_train (pdata->recognizer, pdata->rCharList);
//     pdata->completed=TRUE;
//     g_thread_exit(0);
// }

gboolean trainer_timeout(gpointer data) {
    ProgressData *pdata = (ProgressData *) data;
    gdouble cProgress = writrecogn_stroke_recognizer_get_current_training_progress(progAssoc.recognizer);
    char buf[50];

    g_debug("*** trainer_timeout 1 cProgress=%f\n", cProgress);
    if (cProgress >= 1.0) {
        cProgress = 1.0;
    }
    if (pdata->result==TASK_COMPLETED) {
        cProgress = 1.0;
    }
    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pdata->pbar), cProgress);

    sprintf(buf, "%.4f%% completed", cProgress * 100);
    /* Set the new value */
    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pdata->pbar), buf);
    /* As this is a timeout function, return TRUE so that it
     * continues to get called */
    return (pdata->result==TASK_COMPLETED) ? FALSE : TRUE;
}


TrainingProgressData*
strokeHypothesisFile_start_training_GUI(StrokeRecognizer *recognizer,RadicalList *radicalList) {
    //progAssoc.rCharList=rawCharacterDataFile_get_rawCharacterList();
    //ProgressData *pdata;

    // For GUI_mode
    gint ret=GTK_RESPONSE_CANCEL;
    GtkWidget *labelWidget;
    GtkWidget *pOKButton;
    GtkWidget *pCancelButton;
    gchar *progressBarLabelStr;
    g_debug("strokeHypothesisFile_start_training_GUI\n");

    if (radicalList == NULL) {
	GtkWidget *cantReadDialog = gtk_message_dialog_new(GTK_WINDOW(mainWindowWidget),
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_ERROR,
		GTK_BUTTONS_CLOSE,
		"%s", CHARACTER_DATA_NOT_LOADED_MSG);
	gtk_dialog_run(GTK_DIALOG(cantReadDialog));
	gtk_widget_destroy(cantReadDialog);
	return NULL;
    }
    TrainingProgressData *tProgress = trainingProgressData_new();
    tProgress->pdata.window = progressBarDialogWidget;
    tProgress->pdata.pbar = lookup_widget(progressBarDialogWidget, "progressBar");
    labelWidget = lookup_widget(progressBarDialogWidget, "progressBarLabel");
    pOKButton = lookup_widget(progressBarDialogWidget, "progressBarDialogOkButton");
    pCancelButton = lookup_widget(progressBarDialogWidget, "progressBarDialogCancelButton");

    progressBarLabelStr = g_markup_printf_escaped(LABEL_TEMPLATE, "Generating hypothesis..");
    gtk_label_set_markup(GTK_LABEL(labelWidget), progressBarLabelStr);
    //    g_free(progressBarLabelStr);

    gtk_widget_set_sensitive(pOKButton, FALSE);
    gtk_widget_set_sensitive(pCancelButton, TRUE);


    pthread_create(&trainer_tid, NULL, trainer_thread, (gpointer) tProgress);
    //     GThread *trainerThread=g_thread_create(trainer_thread,(gpointer) tProgress->pdata,TRUE,NULL);
    tProgress->pdata.timer = g_timeout_add_full(G_PRIORITY_DEFAULT, 3000, 
	    trainer_timeout, (gpointer) &tProgress->pdata, strokeHypothesisFile_finish_training);
    while ((ret = gtk_dialog_run(GTK_DIALOG(progressBarDialogWidget))) != GTK_RESPONSE_OK) {
        // Ask whether to cancel the task
        GtkWidget *confirmDialog = gtk_message_dialog_new(GTK_WINDOW(progressBarDialogWidget),
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_ERROR,
                                   GTK_BUTTONS_YES_NO,
                                   "Cancel the hypothesis generation?");
        int subRet = gtk_dialog_run(GTK_DIALOG(confirmDialog));
        gtk_widget_destroy(confirmDialog);
        if (subRet == GTK_RESPONSE_YES) {
	    if (pthread_cancel(trainer_tid) != 0) {
		GtkWidget *cantCancelDialog = gtk_message_dialog_new(GTK_WINDOW(progressBarDialogWidget),
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_ERROR,
			GTK_BUTTONS_CLOSE,
			"Cannot cancel hypothesis generation.");
		gtk_dialog_run(GTK_DIALOG(cantCancelDialog));
		gtk_widget_destroy(cantCancelDialog);
	    }else{
		tProgress->pdata.result = TASK_CANCELED;
		break;
	    }
        }
        // subRet==GTK_RESPONSE_NO
    }

    if (ret == GTK_RESPONSE_OK) {
	// Task completed
	writrecogn_stroke_recognizer_set_hypothesis(progAssoc.recognizer, tProgress->hypo);
	tProgress->pdata.result = TASK_COMPLETED;
	progAssoc.flags.hypothesisModified=TRUE;
    } else {
        // Task cancelled.
     }
    destroy_progressBar(&tProgress->pdata);
    gtk_widget_hide(progressBarDialogWidget);
    return tProgress;
}


void strokeHypothesisFile_finish_training(gpointer data) {
    ProgressData *pdata = (ProgressData *) data;
    GtkWidget *labelWidget = NULL;
    GtkWidget *pOKButton = NULL;
    GtkWidget *pCancelButton = NULL;
    gchar *progressBarLabelStr;

    labelWidget = lookup_widget(progressBarDialogWidget, "progressBarLabel");
    pOKButton = lookup_widget(progressBarDialogWidget, "progressBarDialogOkButton");
    pCancelButton = lookup_widget(progressBarDialogWidget, "progressBarDialogCancelButton");


    progressBarLabelStr = g_markup_printf_escaped(LABEL_TEMPLATE, "Saving Hypothesis.");
    gtk_label_set_markup(GTK_LABEL(labelWidget), progressBarLabelStr);
    g_free(progressBarLabelStr);

    if (pdata->result!=TASK_CANCELED) {
	strokeHypothesisFile_save_GUI(progAssoc.strokeHypothesisFileName);
    }


    progressBarLabelStr = g_markup_printf_escaped(LABEL_TEMPLATE, "Hypothesis generation completed.");
    gtk_label_set_markup(GTK_LABEL(labelWidget), progressBarLabelStr);
    g_free(progressBarLabelStr);

    gtk_widget_set_sensitive(pOKButton, TRUE);
    gtk_widget_set_sensitive(pCancelButton, FALSE);
}


