/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */


/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <unistd.h>
#include <gtk/gtk.h>
#include "interface.h"

#include "support.h"
#include "ui.h"
#include "common.h"
#include "fileProcessing.h"
#include "str_functions.h"


GtkWidget *openFileChooserDialogWidget = NULL;
GtkWidget *saveAsFileChooserDialogWidget = NULL;
GtkWidget *inputCodesDialogWidget = NULL;
GtkWidget *progressBarDialogWidget = NULL;
GtkWidget *mainWindowWidget = NULL;
//canvas=(GtkDrawingArea*) lookup_widget(mainWindow,"canvas");
GtkWidget *canvasWidget = NULL;
GtkWidget *statusBarWidget = NULL;
GtkWidget *wordQueryEntryWidget = NULL;
GtkWidget *candidateEntryWidget = NULL;



void init() {
    progAssoc.choose_open_callback = filename_get_user_chosen_callback_open_GUI;
    progAssoc.choose_save_callback = filename_get_user_chosen_callback_save_GUI;
    progAssoc.training_progress_callback = strokeHypothesisFile_start_training_GUI;

    /*
     * The following code was added by Glade to create one of each component
     * (except popup menus), just so that you see something after building
     * the project. Delete any components that you don't want shown initially.
     */
    openFileChooserDialogWidget = create_openFileChooserDialog();
    saveAsFileChooserDialogWidget = create_openFileChooserDialog();
    inputCodesDialogWidget = create_inputCodesDialog();
    progressBarDialogWidget = create_progressBarDialog();
    mainWindowWidget = create_window();
    canvasWidget = lookup_widget(mainWindowWidget, "canvas");
    statusBarWidget = lookup_widget(mainWindowWidget, "statusBar");
    wordQueryEntryWidget = lookup_widget(mainWindowWidget, "wordQueryEntry");
    candidateEntryWidget = lookup_widget(mainWindowWidget, "candidateEntry");
    canvas_init();

    GtkWidget *treeViewWidget = lookup_widget(inputCodesDialogWidget, "inputCodesDialog_treeView");
    GtkCellRenderer *cellRenderer = gtk_cell_renderer_text_new();
    GtkTreeViewColumn *methodColumn = gtk_tree_view_column_new_with_attributes("Method", cellRenderer, "text", 0, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeViewWidget), methodColumn);
    GtkTreeViewColumn *inputCodeColumn = gtk_tree_view_column_new_with_attributes("Input Code", cellRenderer, "text", 1, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeViewWidget), inputCodeColumn);

    set_displayMode(progAssoc.displayMode);

}

int main(int argc, char *argv[]) {


#ifdef ENABLE_NLS

    bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
    textdomain(GETTEXT_PACKAGE);
#endif

    progAssoc.progCmd = RECOGNIZER;
    common_init(argc, argv);

    /* init threads */
    g_thread_init(NULL);
    gdk_threads_init();
    gdk_threads_enter();

    gtk_set_locale();
    gtk_init(&argc, &argv);

    add_pixmap_directory("pixmaps");
    add_pixmap_directory(PIXMAP_DIR);
    //     add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");

    init();
    progAssoc.flags.initializedFlag=TRUE;
    gtk_widget_show(mainWindowWidget);
    canvas_redraw();

    GString *strBuf = g_string_new(NULL);

    /* Load Character List and Hypothesis if present */
    if (!isEmptyString(progAssoc.characterDataFileName)) {
	progAssoc.characterDataFile=writrecogn_character_datafile_new_from_file(
		    progAssoc.characterDataFileName,FILE_OPEN_TYPE_READ_WRITE);
	if (progAssoc.characterDataFile==NULL){
	    g_string_append_printf(strBuf, "DataFile %s newing failed",
		    progAssoc.characterDataFileName);
	}else if (!writrecogn_character_datafile_open(progAssoc.characterDataFile)){
	    g_string_append_printf(strBuf, "DataFile %s opening failed",
		    progAssoc.characterDataFileName);
	}else{
	    progAssoc.currRadicalList=
		writrecogn_character_datafile_read_radicalList(progAssoc.characterDataFile);
	    if (writrecogn_radical_list_is_empty(progAssoc.currRadicalList)){
		g_string_append_printf(strBuf, "No char loaded",
			progAssoc.characterDataFileName);
	    }else{
		g_string_append_printf(strBuf, "%d chars read, ",
			writrecogn_radical_list_size(progAssoc.currRadicalList));
	    }
	}
    }
    if (!isEmptyString(progAssoc.strokeHypothesisFileName)) {
	if (writrecogn_stroke_recognizer_load_hypothesis
		(progAssoc.recognizer,progAssoc.strokeHypothesisFileName)) {
            g_string_append_printf(strBuf, "Hypothesis loaded.");
        }else{
	    g_string_append_printf(strBuf, "Fail Hypothesis loading.");
	}
    }
    gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, strBuf->str);
    g_string_free(strBuf, TRUE);

    if (queryWord != NULL) {
	gtk_entry_set_text(GTK_ENTRY(wordQueryEntryWidget), queryWord);
	display_queryWord(queryWord);
    }
    windowTitle_update();
    gtk_main();
    gdk_threads_leave();
    return 0;
}
