/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */
#include <gtk/gtk.h>
#include <libgen.h>
#include "ui.h"
#include "WritRecogn-Types.h"
#include "fileProcessing.h"

void revert_word() {
//    g_debug("revert_word \n");
    canvas_set_current_fullCharacter(canvas_get_original_fullCharacter());
    canvas_redraw();
}


void display_queryWord(const char *word) {
    if (isEmptyString(word)) {
        /* Reset the canvas  */
        canvas_reset();
        return;
    }
    FullCharacter *currentChar = NULL;
    char buf[PATH_MAX];
    if (writrecogn_radical_list_is_empty(progAssoc.currRadicalList)) {
        g_snprintf(buf, PATH_MAX, "[Error] Stroke data have not been loaded yet!");
        gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, buf);
        verboseMsgPrint(VERBOSE_MSG_ERROR, "%s\n", buf);
        return;
    }
    /* Treat old char */

    /* Load new char */
    FullCharacter *fChar = FULLCHARACTER(writrecogn_radical_list_find_radical_by_utf8(
	    progAssoc.currRadicalList, word,RADICAL_TYPE_FULLCHARACTER));
    if (fChar == NULL) {
	RadicalCode code=(RadicalCode) g_utf8_get_char(word);
        /* Copy the "word" to the characterCode field of FullCharacter in canvas */
        canvas_reset_current_fullCharacter();
	fChar = canvas_get_current_fullCharacter();
	writrecogn_radical_set_radicalCode(RADICAL(fChar),code);

        canvas_reset_original_fullCharacter();
        fChar = canvas_get_original_fullCharacter();
	writrecogn_radical_set_radicalCode(RADICAL(fChar),code);
        g_snprintf(buf, PATH_MAX, "Word %s is not found. Maybe you should draw one for it.", word);
    } else {
        canvas_set_current_fullCharacter(fChar);
        canvas_set_original_fullCharacter(fChar);
	currentChar=canvas_get_current_fullCharacter();

        g_snprintf(buf, PATH_MAX, "Word %s is found and loaded.", word);
    }

    gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, buf);
    canvas_redraw();
    canvas_set_modified(FALSE);
}



DisplayMode get_displayMode() {
    return progAssoc.displayMode;
}

void set_displayMode(DisplayMode displayMode) {
    progAssoc.displayMode = displayMode;
    GtkWidget *recognizingMenuItemWidget = lookup_widget(mainWindowWidget, "Edit_Recognizing");

    GtkWidget *trainingMenuItemWidget = lookup_widget(mainWindowWidget, "Edit_Training");
    GtkWidget *unihanMenuItemWidget = lookup_widget(mainWindowWidget, "Edit_UniHan");
    GtkWidget *saveMenuItemWidget = lookup_widget(mainWindowWidget, "File_Save");
    GtkWidget *saveAsMenuItemWidget = lookup_widget(mainWindowWidget, "File_SaveAs");

    GtkWidget *inputCodeButtonWidget = lookup_widget(mainWindowWidget, "inputCodeButton");
    GtkWidget *saveButtonWidget = lookup_widget(mainWindowWidget, "saveButton");

    switch (progAssoc.displayMode) {
    case RECOGNIZING_MODE:
        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(recognizingMenuItemWidget), TRUE);
        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(trainingMenuItemWidget), FALSE);
        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(unihanMenuItemWidget), FALSE);


        gtk_widget_set_sensitive(saveMenuItemWidget, FALSE);
        gtk_widget_set_sensitive(saveAsMenuItemWidget, FALSE);
	gtk_widget_set_sensitive(inputCodeButtonWidget, FALSE);
	gtk_widget_set_sensitive(saveButtonWidget, FALSE);
        gtk_widget_set_sensitive(wordQueryEntryWidget, FALSE);

        gtk_entry_set_text(GTK_ENTRY(wordQueryEntryWidget), "");
        canvas_clear();
        gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, "Recognizing Mode Selected.");

        break;
    case TRAINING_MODE:
        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(recognizingMenuItemWidget), FALSE);
        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(trainingMenuItemWidget), TRUE);
        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(unihanMenuItemWidget), FALSE);

        gtk_widget_set_sensitive(saveMenuItemWidget, TRUE);
        gtk_widget_set_sensitive(saveAsMenuItemWidget, TRUE);
	gtk_widget_set_sensitive(inputCodeButtonWidget, TRUE);
	gtk_widget_set_sensitive(saveButtonWidget, TRUE);
        gtk_widget_set_sensitive(wordQueryEntryWidget, TRUE);

        gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, "Training Mode Selected.");
        break;
    case UNIHAN_MODE:
        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(recognizingMenuItemWidget), FALSE);
        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(trainingMenuItemWidget), FALSE);
        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(unihanMenuItemWidget), TRUE);

        gtk_widget_set_sensitive(saveMenuItemWidget, TRUE);
        gtk_widget_set_sensitive(saveAsMenuItemWidget, TRUE);
	gtk_widget_set_sensitive(inputCodeButtonWidget, TRUE);
	gtk_widget_set_sensitive(saveButtonWidget, TRUE);
        gtk_widget_set_sensitive(wordQueryEntryWidget, TRUE);

        gtk_statusbar_push(GTK_STATUSBAR(statusBarWidget), fileStatusContextId, "UniHan Mode Selected.");
        break;
    }
}


/*======================================================
 * For inputCodesDialogWidget
 */
InputCodeRec *gtkTreeIter_to_inputCodeRec(GtkTreeModel *treeModel,GtkTreeIter *tIter) {
    GValue methodValue = { 0 }, iCodeValue = { 0 };
    InputMethod iMethod;
    const gchar *iCode;

    gtk_tree_model_get_value(treeModel, tIter, 0, &methodValue);
    const gchar *methodStr=g_value_get_string(&methodValue);
    if (isEmptyString(methodStr)) {
        return NULL;
    }
    if ((iMethod = inputMethod_parse(methodStr))==IM_INVALID) {
        return NULL;
    }
    gtk_tree_model_get_value(treeModel, tIter, 1, &iCodeValue);
    iCode = g_value_get_string(&iCodeValue);
    if (isEmptyString(iCode)) {
        return NULL;
    }
    return inputCodeRec_new(iMethod,iCode);
}

void windowTitle_update() {
    gchar filenameBuf[PATH_MAX];
    initString(filenameBuf);
    gchar *filename=NULL;
    GString *strBuf=g_string_new("WritRecogn: ");
    if (!isEmptyString(progAssoc.characterDataFileName)){
	strcpy(filenameBuf,progAssoc.characterDataFileName);
	filename=basename(filenameBuf);
	g_string_append(strBuf,filename);
    }else{
	g_string_append(strBuf,"(no file)");
    }
    if (progAssoc.flags.characterDataModified ||
	    ((progAssoc.displayMode!=RECOGNIZING_MODE) && (progAssoc.flags.canvasModified))) {
	g_string_prepend(strBuf,"* ");
    } 
    if (progAssoc.flags.hypothesisModified) {
	g_string_prepend(strBuf,"+ ");
    }
    gtk_window_set_title(GTK_WINDOW(mainWindowWidget), strBuf->str);
    g_string_free(strBuf,TRUE);
}

void characterDataFile_check_saved(){
    int ans;
    if (!progAssoc.characterDataFile)
	return;
    if ( progAssoc.flags.characterDataModified) {
	GtkWidget *msgDialog = gtk_message_dialog_new(GTK_WINDOW(mainWindowWidget),
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_QUESTION,
		GTK_BUTTONS_NONE,
		"Character data are not saved, save before quit?");
	gtk_dialog_add_buttons (GTK_DIALOG(msgDialog),
		GTK_STOCK_SAVE,GTK_RESPONSE_YES,
		GTK_STOCK_NO,GTK_RESPONSE_NO, GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,NULL);
	ans= gtk_dialog_run(GTK_DIALOG(msgDialog));
	gtk_widget_destroy(msgDialog);
	switch(ans){
	    case GTK_RESPONSE_YES:
		if (characterDataFile_save_GUI(progAssoc.characterDataFile)){
		    progAssoc.flags.characterDataModified=FALSE;
		    windowTitle_update();
		}
		break;
	    case GTK_RESPONSE_NO:
		break;
	    case GTK_RESPONSE_CANCEL:
		return;
	}
    }
}

void strokeHypothesisFile_check_saved(){
    int ans;
    Hypothesis *hypo;
    if (!progAssoc.recognizer)
	return;
    if ((hypo=writrecogn_stroke_recognizer_get_hypothesis(progAssoc.recognizer))==NULL)
	return;
    if (progAssoc.flags.hypothesisModified) {
	GtkWidget *msgDialog_hypo = gtk_message_dialog_new(GTK_WINDOW(mainWindowWidget),
		GTK_DIALOG_DESTROY_WITH_PARENT,
		GTK_MESSAGE_QUESTION,
		GTK_BUTTONS_NONE,
		"Hypothesis are not saved, save before quit?");
	gtk_dialog_add_buttons (GTK_DIALOG(msgDialog_hypo),
		GTK_STOCK_SAVE,GTK_RESPONSE_YES,
		GTK_STOCK_NO,GTK_RESPONSE_NO, GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,NULL);
	ans= gtk_dialog_run(GTK_DIALOG(msgDialog_hypo));
	gtk_widget_destroy(msgDialog_hypo);
	switch(ans){
	    case GTK_RESPONSE_YES:
		if (strokeHypothesisFile_save_GUI(progAssoc.strokeHypothesisFileName)){
		    progAssoc.flags.hypothesisModified=FALSE;
		    windowTitle_update();
		}
		break;
	    case GTK_RESPONSE_NO:
		break;
	    case GTK_RESPONSE_CANCEL:
		return;
	}
    }
}

void quitting() {
    characterDataFile_check_saved();
    strokeHypothesisFile_check_saved() ;
    if (progAssoc.characterDataFile){
	writrecogn_character_datafile_close(progAssoc.characterDataFile,FALSE);
    }
    gtk_main_quit();
}

