/*
 * Copyright © 2008  Red Hat, Inc. All rights reserved.
 *  
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc. 
 * 
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com> 
 */

/**
 * Handle Collections such as Hash Set
 *
 */
#ifndef COLLECTION_H_
#define COLLECTION_H_
#include <glib.h>
#define G_ARRAY_COPY(dest,src) g_array_append_vals(dest,src->data,src->len)
#define G_ARRAY_REMOVE_ALL(array) if (array->len>0) g_array_remove_range(array,0,array->len)
#define G_PTR_ARRAY_REMOVE_ALL(array)if (array->len>0) g_ptr_array_remove_range(array,0,array->len)

/**
 * SetType:
 * @INTERGER_SET: Specify that set elements are integers
 * @STRING_SET: Specify that set elements are strings
 * @DIRECT_SET: Specify that set elements are pointers
 * @CUSTOM_SET: Used if non of above fit the need.
 *
 * Specify what will be stored in the set.
 */ 
typedef enum{
    INTEGER_SET,
    STRING_SET,
    DIRECT_SET,
    CUSTOM_SET
} SetType;


typedef struct HashSet_{
    GHashTable *hTable;
    SetType setType;
}HashSet;

HashSet *hashSet_new_auto(SetType type);

HashSet *hashSet_new(SetType type,GHashFunc hash_func, GEqualFunc key_equal_func) ;

HashSet *hashSet_new_full(SetType type,GHashFunc hash_func, GEqualFunc key_equal_func,
             GDestroyNotify key_destroy_func, GDestroyNotify value_destroy_func) ;

void hashSet_reset(HashSet *s);

void hashSet_copy(HashSet *dest, HashSet *src);

guint hashSet_get_size(HashSet *s);

gboolean hashSet_has_element(HashSet *set,gconstpointer key);

GList* hashSet_get_elements(HashSet *s);

/**
 * hashSet_add_element:
 * Add new element to @s, if it does not have the element.
 * 
 * @s the HashSet
 * @key the element to be insert
 */
gboolean hashSet_add_element(HashSet *s,gpointer key);

gboolean hashSet_remove_element(HashSet *s,gconstpointer key);

void hashSet_union(HashSet *result, HashSet *s1, HashSet *s2);

void hashSet_interset(HashSet *result, HashSet *s1, HashSet *s2);

gchar* hashSet_to_string(HashSet *s);

void hashSet_destroy(HashSet *s);

gint integer_compareFunc(gconstpointer a,gconstpointer b);

gint g_array_find(GArray *array,gpointer key, gint keySize,GCompareFunc func);

GArray *g_array_copy(GArray *dest,GArray *src);

#endif /*COLLECTION_H_*/

