/*
 *
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */
#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <gtk/gtk.h>
#include "utils.h"
#include "math_functions.h"


Coordinate2D*
newCoordinate2D(gint x,gint y){
    Coordinate2D *result=NEW_INSTANCE(Coordinate2D);
    if (result!=NULL){
        result->x=x;
        result->y=y;
    }
    return result;
}

Coordinate3D* 
newCoordinate3D(gint x,gint y,gint z){
    Coordinate3D *result=NEW_INSTANCE(Coordinate3D);
    if (result!=NULL){
        result->x=x;
        result->y=y;
        result->z=z;
    }
    return result;
}

void 
coordinate2D_copy(Coordinate2D *dest,Coordinate2D *src){
    coordinate2D_set(dest,src->x,src->y);
}

void 
coordinate3D_copy(Coordinate3D *dest,Coordinate3D *src){
    coordinate3D_set(dest,src->x,src->y,src->z);
}

void 
coordinate2D_set(Coordinate2D *coord,gint x,gint y){
    coord->x=x;
    coord->y=y;
}

void
coordinate3D_set(Coordinate3D *coord,gint x,gint y,gint z){
    coord->x=x;
    coord->y=y;
    coord->z=z;
}


MathVector2D *
coordinate2D_to_mathVector2D(MathVector2D *vec,Coordinate2D *start,Coordinate2D *end){
    return mathVector2D_minus(vec,end,start);
}

MathVector3D *
coordinate3D_to_mathVector3D(MathVector3D *vec,Coordinate3D *start,Coordinate3D *end){
    return mathVector3D_minus(vec,end,start);
}

gchar *
coordinate2D_to_string(Coordinate2D *coord){
    GString *gStr=g_string_new("");
    g_string_printf(gStr,"x=%d,y=%d",coord->x,coord->y);
    return g_string_free(gStr,FALSE);
}

gchar *
coordinate3D_to_string(Coordinate3D *coord){
    GString *gStr=g_string_new("");
    g_string_printf(gStr,"x=%d,y=%d,z=%d",coord->x, coord->y,coord->z);
    return g_string_free(gStr,FALSE);
}

gdouble 
coordinate2D_cosine(Coordinate2D *coord1, Coordinate2D *angleCoord, Coordinate2D *coord2){
    MathVector2D v1,v2;
    coordinate2D_to_mathVector2D(&v1, angleCoord,coord1);
    coordinate2D_to_mathVector2D(&v2, angleCoord,coord2);
    return mathVector2D_cosine(&v1,&v2);
}

gdouble 
coordinate3D_cosine(Coordinate3D *coord1, Coordinate3D *angleCoord, Coordinate3D *coord2){
    MathVector3D v1,v2;
    coordinate3D_to_mathVector3D(&v1, angleCoord,coord1);
    coordinate3D_to_mathVector3D(&v2, angleCoord,coord2);
    return mathVector3D_cosine(&v1,&v2);
}

Coordinate2D*
coordinate2D_midpoint(Coordinate2D *result, Coordinate2D *c1, Coordinate2D *c2){
    result->x=(c1->x+c2->x)/2;
    result->y=(c1->y+c2->y)/2;
    return result;
}

Coordinate3D*
coordinate3D_midpoint(Coordinate3D *result, Coordinate3D *c1, Coordinate3D *c2){
    result->x=(c1->x+c2->x)/2;
    result->y=(c1->y+c2->y)/2;
    result->y=(c1->z+c2->z)/2;
    return result;
}

gdouble 
triangle2D_area_given_Coordinate2D(Coordinate2D *c1, Coordinate2D *c2, Coordinate2D *c3) {
    Coordinate3D c1_3D,c2_3D,c3_3D;
    MathVector3D result;
    mathVector2D_to_mathVector3D(&c1_3D,c1);
    mathVector2D_to_mathVector3D(&c2_3D,c2);
    mathVector2D_to_mathVector3D(&c3_3D,c3);
    vectorPoint_crossProduct(&result,&c1_3D,&c2_3D,&c1_3D,&c3_3D);
    return mathVector3D_EuclideanNorm(&result)/2;
}

gdouble 
triangle2D_area_given_points(gint p1_x,gint p1_y, gint p2_x, gint p2_y, gint p3_x, gint p3_y) {
    Coordinate2D c1,c2,c3;
    c1.x=p1_x; c1.y=p1_y;
    c2.x=p2_x; c2.y=p2_y;
    c3.x=p3_x; c3.y=p3_y;
    return triangle2D_area_given_Coordinate2D(&c1,&c2,&c3);
}


gdouble
distance2D_between_point_and_line(Coordinate2D *p, Coordinate2D *l1, Coordinate2D *l2) {
    Coordinate3D p_3D,l1_3D,l2_3D;
    MathVector3D result;
    mathVector2D_to_mathVector3D(&p_3D,p);
    mathVector2D_to_mathVector3D(&l1_3D,l1);
    mathVector2D_to_mathVector3D(&l2_3D,l2);
    vectorPoint_crossProduct(&result,&l1_3D,&p_3D,&l1_3D,&l2_3D);
//     printf("***result %s \n",coordinate3D_to_string(&result));
    gdouble area=mathVector3D_EuclideanNorm(&result);
    gdouble lineLen=distance2D_between_points(l1,l2);
//     printf("***  area=%f lineLen=%f \n",area,lineLen);
    return area/lineLen;
}

gdouble
distance2D_between_points(Coordinate2D *l1, Coordinate2D *l2) {
    Coordinate3D l1_3D,l2_3D;
    MathVector3D lineVec;
    mathVector2D_to_mathVector3D(&l1_3D,l1);
    mathVector2D_to_mathVector3D(&l2_3D,l2);
    coordinate3D_to_mathVector3D(&lineVec,&l1_3D,&l2_3D);
    return mathVector3D_EuclideanNorm(&lineVec);
}

/*===================================================================
 * MathVector functions
 */

MathVector3D*
mathVector2D_to_mathVector3D(MathVector3D *vec,MathVector2D *src){
    vec->x=src->x;
    vec->y=src->y;
    vec->z=0;
    return vec;
}

MathVector2D*
mathVector2D_plus(MathVector2D *result, MathVector2D *v1, MathVector2D *v2){
    result->x=v1->x+v2->x;
    result->y=v1->y+v2->y;
    return result;
}

MathVector3D*
mathVector3D_plus(MathVector3D *result, MathVector3D *v1, MathVector3D *v2){
    result->x=v1->x+v2->x;
    result->y=v1->y+v2->y;
    result->z=v1->z+v2->z;
    return result;
}


MathVector2D*
mathVector2D_minus(MathVector2D *result, MathVector2D *v1, MathVector2D *v2){
    result->x=v1->x-v2->x;
    result->y=v1->y-v2->y;
    return result;
}

MathVector3D*
mathVector3D_minus(MathVector3D *result, MathVector3D *v1, MathVector3D *v2){
    result->x=v1->x-v2->x;
    result->y=v1->y-v2->y;
    result->z=v1->z-v2->z;
    return result;
}

MathVector2D*
mathVector2D_scale(MathVector2D *result,MathVector2D *v,gdouble scale){
    result->x=DOUBLE_TO_INT( v->x * scale);
    result->y=DOUBLE_TO_INT( v->y * scale);
    return result;
}

MathVector3D*
mathVector3D_scale(MathVector3D *result,MathVector3D *v,gdouble scale){
    result->x=DOUBLE_TO_INT( v->x * scale);
    result->y=DOUBLE_TO_INT( v->y * scale);
    result->z=DOUBLE_TO_INT( v->z * scale);
    return result;
}

MathVector2D*
mathVector2D_project(MathVector2D *result, MathVector2D *v1, MathVector2D *v2){
    gint dotProduct = mathVector2D_dotProduct(v1, v2);
    gdouble v1Len=mathVector2D_EuclideanNorm(v1);
    /* 
     * cos=dotProduct/v1Len/v2Len
     * scale=v2Len*cos/v1Len
     */
    gdouble scale= dotProduct/v1Len/v1Len;
//    g_debug("mathVector2D_project dotProduct=%d v1Len=%f v2Len=%f, scale=%f\n",dotProduct,v1Len,v2Len,scale);
    coordinate2D_copy(result,v1);
    mathVector2D_scale(result,result,scale);
    return result;
}

MathVector3D*
mathVector3D_project(MathVector3D *result, MathVector3D *v1, MathVector3D *v2){
    gint dotProduct = mathVector3D_dotProduct(v1, v2);
    gdouble v1Len=mathVector3D_EuclideanNorm(v1);
    gdouble scale=dotProduct/v1Len/v1Len;
    coordinate3D_copy(result,v1);
    mathVector3D_scale(result,result,scale);
    return result;
}


gboolean
mathVector2D_is_counterclockwise(MathVector2D *src, MathVector2D *target){
    MathVector3D v1_3D,v2_3D,result;
    mathVector2D_to_mathVector3D(&v1_3D,src);
    mathVector2D_to_mathVector3D(&v2_3D,target);
    vector_crossProduct(&result,&v1_3D,&v2_3D);
    if (result.z>0){
        return TRUE;
    }
    return FALSE;
}

gdouble 
mathVector2D_sine(MathVector2D *v1, MathVector2D *v2){
    MathVector3D v1_3D,v2_3D;
    mathVector2D_to_mathVector3D(&v1_3D,v1);
    mathVector2D_to_mathVector3D(&v2_3D,v2);
    return mathVector3D_sine(&v1_3D,&v2_3D);
}


gdouble 
mathVector3D_sine(MathVector3D *v1, MathVector3D *v2){
    MathVector3D result;
    vector_crossProduct(&result,v1,v2);
    gdouble v1_len=mathVector3D_EuclideanNorm(v1);
    gdouble v2_len=mathVector3D_EuclideanNorm(v2);
    gdouble result_len=mathVector3D_EuclideanNorm(&result);
    //printf("****  result_len=%.4f v1_len=%.4f v2_len=%.4f \n",result_len, v1_len, v2_len);
    return result_len/v1_len/v2_len;
}

gdouble 
mathVector2D_cosine(MathVector2D *v1, MathVector2D *v2){
    gdouble result_len=mathVector2D_dotProduct(v1,v2);
    gdouble v1_len=mathVector2D_EuclideanNorm(v1);
    gdouble v2_len=mathVector2D_EuclideanNorm(v2);
    return result_len/v1_len/v2_len;
}


gdouble 
mathVector3D_cosine(MathVector3D *v1, MathVector3D *v2){
    gdouble result_len=mathVector3D_dotProduct(v1,v2);
    gdouble v1_len=mathVector3D_EuclideanNorm(v1);
    gdouble v2_len=mathVector3D_EuclideanNorm(v2);
    return result_len/v1_len/v2_len;
}

gint
mathVector2D_dotProduct(MathVector2D *v1, MathVector2D *v2) {
    gint result= v1->x*v2->x;
    result+= v1->y*v2->y;
//    g_debug("mathVector2D_dotProduct=%d\n",result);
    return result;
}

gint
mathVector3D_dotProduct(MathVector3D *v1, MathVector3D *v2) {
    gint result= v1->x*v2->x;
    result+= v1->y*v2->y;
    result+= v1->z*v2->z;
    return result;
}


gdouble 
mathVector2D_includedAngle(MathVector2D *v1, MathVector2D *v2){
    gdouble cosine=mathVector2D_cosine(v1, v2);
    gdouble angle=acos(cosine);
    if (mathVector2D_is_counterclockwise(v1,v2)){
        return angle;
    }
    return -angle;
}

gdouble 
mathVector3D_includedAngle(MathVector3D *v1, MathVector3D *v2){
    gdouble cosine=mathVector3D_cosine(v1, v2);
    return acos(cosine);
}


MathVector3D *
vector_crossProduct(MathVector3D *result, MathVector3D *v1, MathVector3D *v2) {
    result->x=v1->y *v2->z -v1->z *v2->y;
    result->y=v1->z *v2->x -v1->x *v2->z;
    result->z=v1->x *v2->y -v1->y *v2->x;
    // char strbuf1[30],strbuf2[30],strbuf3[30];
    // printf("****  result=(%s) v1=(%s) v2=(%s)\n",coordinate3D_to_string(strbuf1,result),coordinate3D_to_string(strbuf2,v1),coordinate3D_to_string(strbuf3,v2));
    return result;
}

MathVector3D *
vectorPoint_crossProduct(MathVector3D *result,Coordinate3D *v1_start, Coordinate3D * v1_end, Coordinate3D *v2_start, Coordinate3D *v2_end) {
    MathVector3D v1, v2;
    coordinate3D_to_mathVector3D(&v1,v1_start,v1_end);
    coordinate3D_to_mathVector3D(&v2,v2_start,v2_end);
/*    printf("*** v1=%s\t v2=%s\n",coordinate3D_to_string(&v1),coordinate3D_to_string(&v2));*/
    return vector_crossProduct(result,&v1,&v2);
}


gdouble 
mathVector2D_EuclideanNorm(MathVector2D *vec){
    gdouble result=vec->x* vec->x;
    result+=  vec->y* vec->y;
    return sqrt(result);
}

gdouble 
mathVector3D_EuclideanNorm(MathVector3D *vec){
    gdouble result= vec->x*vec->x;
    result+=vec->y*vec->y;
    result+= vec->z*vec->z;
    return sqrt(result);
}

/*===================================================================
 * BoundingBox functions
 */
void
mathBox2D_set(MathBox2D* box,gint left,gint top, gint right,gint bottom){
    box->left=left;
    box->right=right;
    box->top=top;
    box->bottom=bottom;
}


void
mathBox2D_copy(MathBox2D* dest,MathBox2D* src){
    dest->left=src->left;
    dest->right=src->right;
    dest->top=src->top;
    dest->bottom=src->bottom;
}


gchar*
mathBox2D_to_string(MathBox2D* box){
    GString *strBuf=g_string_new("");
    g_string_printf(strBuf,"(%d,%d,%d,%d)",box->left,box->top,box->right,box->bottom);
    return g_string_free(strBuf,FALSE);
}

MathBox2D*
mathBox2D_from_string(MathBox2D* box,const gchar *str){
    gchar **resultStr=g_strsplit(str,"(,)",0);
    int i,j=0;
    for(i=0;resultStr[i]!=NULL;i++){
        if(strlen(resultStr[i])==0){
            continue;
        }
        if (j>3){
            break;
        }
        errno=0;
        int result=atoi(resultStr[i]);
        if (errno!=0){
            g_strfreev (resultStr);
            return NULL;
        }
        switch(j){
            case 0:
                box->left=result;
                j++;
                break;
            case 1:
                box->top=result;
                j++;
                break;
            case 2:
                box->right=result;
                j++;
                break;
            case 3:
                box->bottom=result;
                j++;
                break;
        }
    }
    g_strfreev (resultStr);
    return box;
}

MathBox2D *mathBox2D_union(MathBox2D *resultBox,MathBox2D *box1,MathBox2D *box2){
    if (box2->left<0){
        mathBox2D_copy(resultBox,box1);
        return resultBox;
    }
    if (box1->left<0){
        mathBox2D_copy(resultBox,box2);
        return resultBox;
    }
    resultBox->left=(box1->left < box2->left) ? box1->left : box2->left;
    resultBox->top=(box1->top < box2->top) ? box1->top : box2->top;
    resultBox->right=(box1->right > box2->right) ? box1->right : box2->right;
    resultBox->bottom=(box1->bottom > box2->bottom) ? box1->bottom : box2->bottom;
    return resultBox;
}
