/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *  
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc. 
 * 
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com> 
 */

/**
 * Definitions XML IO functions
 */
#ifndef XML_HANDLER_H_
#define XML_HANDLER_H_
#include <stdio.h>
#include <libxml/tree.h>
#include <libxml/xmlreader.h>
#include "verboseMsg.h"

#define XmlNodeType_None 0
#define XmlNodeType_Element 1
#define XmlNodeType_Attribute 2
#define XmlNodeType_Text 3
#define XmlNodeType_CDATA 4
#define XmlNodeType_EntityReference 5
#define XmlNodeType_Entity 6
#define XmlNodeType_ProcessingInstruction 7
#define XmlNodeType_Comment 8
#define XmlNodeType_Document 9
#define XmlNodeType_DocumentType 10
#define XmlNodeType_DocumentFragment 11
#define XmlNodeType_Notation 12
#define XmlNodeType_Whitespace 13
#define XmlNodeType_SignificantWhitespace 14
#define XmlNodeType_EndElement 15
#define XmlNodeType_EndEntity 16
#define XmlNodeType_XmlDeclaration 17

const char *XmlNodeTypeStrings[18];

typedef struct XML_Node_{
	int	nodeType;
	const xmlChar *name;
	const xmlChar *value;
	int   attributeCount;
    xmlAttr *attributeList;
        //    const xmlChar **attributeNames;
        //    const xmlChar **attributeValues;
} XML_Node;

/**
 * XML callback function definitions.
 */

typedef void (* XML_Node_iteration_func) (xmlTextReaderPtr reader,XML_Node *xml_node, void *userdata);

/**
 * Open the specified XML file 
 * 
 * @param filename the file name to read and parse
 * @return xmlTextReader file point to the opened file. 
 */
xmlTextReaderPtr
openXML_File(const char *filename);

/**
 * Parse from XML Reader 
 * 
 * @param reader the pointer of opened xmlTextReader
 * @return 0 if no error, non-zero otherwise 
 */
int
parseXML(xmlTextReaderPtr reader,XML_Node_iteration_func parseCallback, void *userdata);

/**
 * Convert nodeType id to the discription
 * @param nodeType type id
 */
const char*
xmlNodeTypeToString(int nodeType);

/**
 * Get a node from a open xml file 
 * 
 * @param reader the xmlTextReader
 * @return the parsed XML_Node 
 */
XML_Node*
getXML_Node(xmlTextReaderPtr reader);

/**
 * Parsea node from a open xml file 
 * 
 * @param reader the xmlTextReader
 * @param xml_node the XML_Node to be parsed.
 */
void
XML_Node_parse(xmlTextReaderPtr reader, XML_Node *xml_node, void *userdata);


void XML_Node_set_parse_callback(XML_Node_iteration_func callback);


void XML_Node_dump_info(xmlTextReaderPtr reader,XML_Node *xml_node, void *userdata);
#endif /*XML_HANDLER_H_*/
