/* Generated by GOB (v2.0.15)   (do not edit directly) */



/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

#include <glib.h>
#include <glib-object.h>


/**
 * SECTION: writRecognRadical
 * @short_description: Radical, a basic component of characters.
 * @stability: Stable
 * @see_also: #writRecognAbsCharacter, #writRecognRadical_list
 * @include: writrecogn-radical.h
 *
 * GOB2 generate the class name as <code>writRecognRadical</code>,
 * Use alias <b>Radical</b> for convenience.
 *
 * Radicals are character roots, which forms CJK character.
 * Strokes are treated as lowest level radical.
 * 
 * Radicals should be designed as lightweight, 
 * more "advance" functions such as bounding box should go to 
 * Abstract Character class (writrecogn-abscharacter.gob)
 */

#ifndef RADICAL_H_
#define RADICAL_H_
#include "utils.h"
#include "WritRecogn.h"


/* Wubi stroke types*/
typedef enum {
    UNKNOWN_RADICAL_ELEMENT=0, ///< Stroke that cannot be 
    H_LINE_RADICAL_ELEMENT=-1, ///< Horizontal line
    V_LINE_RADICAL_ELEMENT=-2, ///< Vertical line
    SLASH_RADICAL_ELEMENT=-3, ///< Forward slash
    BACK_SLASH_RADICAL_ELEMENT=-4, ///< Back slash and Dot
    HOOK_RADICAL_ELEMENT=-5 ///< banded line
} StrokeCategoryCategory;

extern gchar *StrokeCategoryNames[];

#endif /* RADICAL_H_ */

/**
 * writRecognRadical:
 * @radicalCode: ID number of Radical. 
 *      If unicode counterpart is present, then the value is set to its unicode value.
 *      Otherwise the radical is indexed as a negative value.
 * @absoluteBoundingBox: Absolute radical bounding box.
 *      The actual radical bounding box is the bounding box in actual coordinate system.
 *      (As in canvas).
 * @relativeBoundingBox: Relative radical Bounding box.
 *      The relative radical bounding box is the bounding box in relative coordinate system.
 *      In relative coordinate system, every radical bounding boxes project onto X and Y axis
 *      as line segments. 
 *      For X axis, the leftmost end point of line segments is indexed as 1,
 *      the 2nd leftmost end point is indexed as 2, and so on.
 *      Similarly, for Y axis, the top 1 end point of line segments is indexed as 1,
 *      the top 2 end point is indexed as 2, and so on.
 *  @isLeaf: whether it has sub radicals. 
 *      TRUE if no sub radical (implies subRadicalArray size 0), it is the default value.
 *      FLASE otherwise.
 *  @subRadicalArray: (protected) a #RadicalArray that actually holds sub-radicals. 
 */

#ifndef __WRITRECOGN_RADICAL_H__
#define __WRITRECOGN_RADICAL_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*
 * Type checking and casting macros
 */
#define WRITRECOGN_TYPE_RADICAL	(writrecogn_radical_get_type())
#define WRITRECOGN_RADICAL(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), writrecogn_radical_get_type(), writRecognRadical)
#define WRITRECOGN_RADICAL_CONST(obj)	G_TYPE_CHECK_INSTANCE_CAST((obj), writrecogn_radical_get_type(), writRecognRadical const)
#define WRITRECOGN_RADICAL_CLASS(klass)	G_TYPE_CHECK_CLASS_CAST((klass), writrecogn_radical_get_type(), writRecognRadicalClass)
#define WRITRECOGN_IS_RADICAL(obj)	G_TYPE_CHECK_INSTANCE_TYPE((obj), writrecogn_radical_get_type ())

#define WRITRECOGN_RADICAL_GET_CLASS(obj)	G_TYPE_INSTANCE_GET_CLASS((obj), writrecogn_radical_get_type(), writRecognRadicalClass)

/*
 * Main object structure
 */
#ifndef __TYPEDEF_WRITRECOGN_RADICAL__
#define __TYPEDEF_WRITRECOGN_RADICAL__
typedef struct _writRecognRadical writRecognRadical;
#endif
struct _writRecognRadical {
	GObject __parent__;
	/*< public >*/
	RadicalCode radicalCode;
	gboolean isLeaf;
	BoundingBox absoluteBoundingBox;
	BoundingBox relativeBoundingBox;
	/*< private >*/
	RadicalArray * subRadicalArray; /* protected */
};

/*
 * Class definition
 */
typedef struct _writRecognRadicalClass writRecognRadicalClass;
struct _writRecognRadicalClass {
	GObjectClass __parent__;
	Radical * (* clone) (writRecognRadical * self);
	void (* copy) (writRecognRadical * self, Radical * src);
	void (* reset) (writRecognRadical * self);
	void (* add_subRadical) (writRecognRadical * self, Radical * subRadical);
	gchar * (* to_string) (writRecognRadical * self);
};


/*
 * Public methods
 */
GType	writrecogn_radical_get_type	(void);
glong 	writrecogn_radical_get_radicalCode	(writRecognRadical * self);
void 	writrecogn_radical_set_radicalCode	(writRecognRadical * self,
					glong val);
gboolean 	writrecogn_radical_get_isLeaf	(writRecognRadical * self);
void 	writrecogn_radical_set_isLeaf	(writRecognRadical * self,
					gboolean val);
Radical * 	writrecogn_radical_new	(void);
Radical * 	writrecogn_radical_new_by_code	(RadicalCode code);
Radical * 	writrecogn_radical_new_by_code_from_radicalList	(RadicalCode code,
					RadicalList * radList);
Radical * 	writrecogn_radical_clone	(writRecognRadical * self);
void 	writrecogn_radical_copy	(writRecognRadical * self,
					Radical * src);
void 	writrecogn_radical_reset	(writRecognRadical * self);
gunichar 	writrecogn_radical_get_radicalCode_ucs4	(writRecognRadical * self);
void 	writrecogn_radical_set_radicalCode_ucs4	(writRecognRadical * self,
					gunichar ucs4Code);
gchar * 	writrecogn_radical_get_radicalCode_utf8	(writRecognRadical * self);
void 	writrecogn_radical_set_radicalCode_utf8	(writRecognRadical * self,
					const gchar * utf8Code);
BoundingBox * 	writrecogn_radical_get_absoluteBoundingBox	(writRecognRadical * self);
void 	writrecogn_radical_set_absoluteBoundingBox	(writRecognRadical * self,
					BoundingBox * box);
BoundingBox * 	writrecogn_radical_get_relativeBoundingBox	(writRecognRadical * self);
void 	writrecogn_radical_set_relativeBoundingBox	(writRecognRadical * self,
					BoundingBox * box);
gchar * 	writrecogn_radical_absoluteBoundingBox_to_string	(writRecognRadical * self);
gchar * 	writrecogn_radical_relativeBoundingBox_to_string	(writRecognRadical * self);
gchar * 	writrecogn_radical_absoluteBoundingBox_to_string_recursively	(writRecognRadical * self);
gchar * 	writrecogn_radical_relativeBoundingBox_to_string_recursively	(writRecognRadical * self);
guint 	writrecogn_radical_count_subRadicals	(writRecognRadical * self);
Radical * 	writrecogn_radical_get_subRadical	(writRecognRadical * self,
					guint i);
void 	writrecogn_radical_add_subRadical	(writRecognRadical * self,
					Radical * subRadical);
gchar * 	writrecogn_radical_to_string	(writRecognRadical * self);

/*
 * Argument wrapping macros
 */
#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#define WRITRECOGN_RADICAL_PROP_RADICALCODE(arg)    	"radicalCode", __extension__ ({glong z = (arg); z;})
#define WRITRECOGN_RADICAL_GET_PROP_RADICALCODE(arg)	"radicalCode", __extension__ ({glong *z = (arg); z;})
#define WRITRECOGN_RADICAL_PROP_ISLEAF(arg)    	"isLeaf", __extension__ ({gboolean z = (arg); z;})
#define WRITRECOGN_RADICAL_GET_PROP_ISLEAF(arg)	"isLeaf", __extension__ ({gboolean *z = (arg); z;})
#define WRITRECOGN_RADICAL_PROP_SUBRADICALARRAY(arg)    	"subRadicalArray", __extension__ ({gpointer z = (arg); z;})
#define WRITRECOGN_RADICAL_GET_PROP_SUBRADICALARRAY(arg)	"subRadicalArray", __extension__ ({gpointer *z = (arg); z;})
#else /* __GNUC__ && !__STRICT_ANSI__ */
#define WRITRECOGN_RADICAL_PROP_RADICALCODE(arg)    	"radicalCode",(glong )(arg)
#define WRITRECOGN_RADICAL_GET_PROP_RADICALCODE(arg)	"radicalCode",(glong *)(arg)
#define WRITRECOGN_RADICAL_PROP_ISLEAF(arg)    	"isLeaf",(gboolean )(arg)
#define WRITRECOGN_RADICAL_GET_PROP_ISLEAF(arg)	"isLeaf",(gboolean *)(arg)
#define WRITRECOGN_RADICAL_PROP_SUBRADICALARRAY(arg)    	"subRadicalArray",(gpointer )(arg)
#define WRITRECOGN_RADICAL_GET_PROP_SUBRADICALARRAY(arg)	"subRadicalArray",(gpointer *)(arg)
#endif /* __GNUC__ && !__STRICT_ANSI__ */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
